@@ -12,12 +12,12 @@ assign("nameEx",
 ## Add some hooks to label plot pages for base and grid graphics
 assign("base_plot_hook",
        function() {
-           pp <- par(c("mfg","mfcol","oma","mar"))
+           pp <- graphics::par(c("mfg","mfcol","oma","mar"))
            if(all(pp$mfg[1:2] == c(1, pp$mfcol[2]))) {
                outer <- (oma4 <- pp$oma[4]) > 0; mar4 <- pp$mar[4]
-               mtext(sprintf("help(\"%s\")", nameEx()), side = 4,
+               graphics::mtext(sprintf("help(\"%s\")", nameEx()), side = 4,
                      line = if(outer)max(1, oma4 - 1) else min(1, mar4 - 1),
-               outer = outer, adj = 1, cex = .8, col = "orchid", las=3)
+               outer = outer, adj = 1, cex = 0.8, col = "orchid", las = 3)
            }
        },
        pos = "CheckExEnv")
@@ -41,26 +41,45 @@ assign("cleanEx",
 	   set.seed(1)
    	   options(warn = 1)
 	   .CheckExEnv <- as.environment("CheckExEnv")
-	   delayedAssign("T", stop("T used instead of TRUE"),
+	   delayedAssign("T", stop("T used instead of TRUE", domain = NA),
 		  assign.env = .CheckExEnv)
-	   delayedAssign("F", stop("F used instead of FALSE"),
+	   delayedAssign("F", stop("F used instead of FALSE", domain = NA),
 		  assign.env = .CheckExEnv)
 	   sch <- search()
 	   newitems <- sch[! sch %in% .oldSearch]
            if(length(newitems)) tools:::detachPackages(newitems)
 	   missitems <- .oldSearch[! .oldSearch %in% sch]
 	   if(length(missitems))
-	       warning("items ", paste(missitems, collapse=", "),
-		       " have been removed from the search path")
+	       warning(sprintf("items %s were removed from the search path",
+                               paste(sQuote(missitems), collapse=", ")),
+                       call. = FALSE, immediate. = TRUE, domain = NA)
+           ## Old massaged files will not have set .old_wd.
+           if(exists(".old_wd") && (wd <- getwd()) != .old_wd) {
+               warning(sprintf("working directory was changed to %s, resetting",
+                               sQuote(wd)),
+                       call. = FALSE, immediate. = TRUE, domain = NA)
+               setwd(.old_wd)
+           }
+           ## stop in case users left connections open,
+           ## also indicating that parallel cluster are still running
+           if(Sys.getenv("_R_CHECK_CONNECTIONS_LEFT_OPEN_", FALSE)){
+               sC <- showConnections()
+               if(nrow(sC)){
+                   stop("connections left open:\n",
+                       paste(apply(sC[,1:2, drop = FALSE], 1L, function(x)
+                           paste0("\t", x[1L], " (", x[2L], ")")), collapse="\n"),
+				       call. = FALSE, domain = NA)
+               }
+           }
        },
        pos = "CheckExEnv")
 assign("ptime", proc.time(), pos = "CheckExEnv")
-## at least one package changes these via ps.options(), so do this
-## before loading the package.
-## Use postscript as incomplete files may be viewable, unlike PDF.
-## Choose a size that is close to on-screen devices, fix paper
-grDevices::ps.options(width = 7, height = 7, paper = "a4", reset = TRUE)
-grDevices::postscript(paste(pkgname, "-Ex.ps", sep=""), encoding = "ISOLatin1")
+## Do this before loading the package,
+## since packages have been known to change settings.
+## Force a size that is close to on-screen devices, fix paper.
+## don't rename par.postscript for back-compatibility of reference output.
+grDevices::pdf.options(width = 7, height = 7, paper = "special", reset = TRUE)
+grDevices::pdf(paste(pkgname, "-Ex.pdf", sep=""), encoding = "ISOLatin1")
 
 assign("par.postscript", graphics::par(no.readonly = TRUE), pos = "CheckExEnv")
 options(contrasts = c(unordered = "contr.treatment", ordered = "contr.poly"))
