@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2014   The R Core Team.
+ *  Copyright (C) 1997-2018   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,9 +15,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
+/* Internal header, not installed */
+
 #ifndef PRINT_H_
 #define PRINT_H_
 
@@ -35,29 +37,17 @@
 #define printNamedVector    Rf_printNamedVector
 #define printVector         Rf_printVector
 
-typedef struct {
-    int width;
-    int na_width;
-    int na_width_noquote;
-    int digits;
-    int scipen;
-    int gap;
-    int quote;
-    int right;
-    int max;
-    SEXP na_string;
-    SEXP na_string_noquote;
-    int useSource;
-    int cutoff; // for deparsed language objects
-} R_print_par_t;
+/* For backward compatibility */
+#define R_print_par_t R_PrintData
+
 extern R_print_par_t R_print;
 
 /* Computation of printing formats */
-void formatRaw(Rbyte *, R_xlen_t, int *);
-void formatString(SEXP*, R_xlen_t, int*, int);
+void formatRaw(const Rbyte *, R_xlen_t, int *);
+void formatString(const SEXP*, R_xlen_t, int*, int);
 
 /* Formating of values */
-const char *EncodeElement0(SEXP, int, int, const char *);
+const char *EncodeElement0(SEXP, R_xlen_t, int, const char *);
 const char *EncodeEnvironment(SEXP);
 /* Legacy, for R.app */
 const char *EncodeElement(SEXP, int, int, char);
