@@ -15,7 +15,7 @@
 #  https://www.R-project.org/Licenses/
 
 # Statlib code by John Chambers, Bell Labs, 1994
-# Changes Copyright (C) 1998-2016 The R Core Team
+# Changes Copyright (C) 1998-2019 The R Core Team
 
 
 ## As from R 2.4.0, row.names can be either character or integer.
@@ -43,9 +43,14 @@ row.names.default <- function(x) if(!is.null(dim(x))) rownames(x)# else NULL
 .set_row_names <- function(n)
     if(n > 0) c(NA_integer_, -n) else integer()
 
+
+##_H Hack around the fact that other packages fail with a newly improved `row.names<-`:
+##_H
+##_H `row.names<-` <- function(x, make.names = FALSE, value) UseMethod("row.names<-")
 `row.names<-` <- function(x, value) UseMethod("row.names<-")
 
-`row.names<-.data.frame` <- function(x, value)
+##_H `row.names<-.data.frame` <-
+`.rowNamesDF<-` <- function(x, make.names = FALSE, value)
 {
     if (!is.data.frame(x)) x <- as.data.frame(x)
     n <- .row_names_info(x, 2L)
@@ -62,24 +67,58 @@ row.names.default <- function(x) if(!is.null(dim(x))) rownames(x)# else NULL
         if(!is.null(attr(x, "row.names")) && length(value) > 0L)
            stop("invalid 'row.names' length")
     }
-    else if (length(value) != n)
-	stop("invalid 'row.names' length")
+    else if (length(value) != n) {
+	if(isFALSE(make.names)) stop("invalid 'row.names' length")
+        else if(is.na(make.names)) { # automatic row.names
+            attr(x, "row.names") <- .set_row_names(n)
+            return(x)
+        }
+        else if(!isTRUE(make.names)) stop("invalid 'make.names'")
+        ## else  make.names = TRUE: amend 'value' to correct ones:
+        else if((nv <- length(value)) < n)
+            value <- c(value, rep_len(value[nv], n-nv))
+        else # length(value) > n
+            value <- value[seq_len(n)]
+    }
     if (anyDuplicated(value)) {
-        nonuniq <- sort(unique(value[duplicated(value)]))
-        warning(ngettext(length(nonuniq),
-                         sprintf("non-unique value when setting 'row.names': %s",
-                                 sQuote(nonuniq[1L])),
-                         sprintf("non-unique values when setting 'row.names': %s",
-                                 paste(sQuote(nonuniq), collapse = ", "))),
+        if(isFALSE(make.names)) {
+            nonuniq <- sort(unique(value[duplicated(value)]))
+            ## warning + stop ?? FIXME: s/warning/stop/ and drop (2nd) stop ??
+            warning(ngettext(length(nonuniq),
+                             sprintf("non-unique value when setting 'row.names': %s",
+                                     sQuote(nonuniq[1L])),
+                             sprintf("non-unique values when setting 'row.names': %s",
+                                     paste(sQuote(nonuniq), collapse = ", "))),
                 domain = NA, call. = FALSE)
-	stop("duplicate 'row.names' are not allowed")
+            stop("duplicate 'row.names' are not allowed")
+        }
+        else if(is.na(make.names)) { # automatic row.names
+            value <- .set_row_names( # find nrow(.) in case 'n' is not usable:
+                if(n == 0L && is.null(.row_names_info(x, 0L)) && length(x) > 0L)
+                    length(x[[1L]])
+                else n)
+        }
+        else if(!isTRUE(make.names)) stop("invalid 'make.names'")
+        else # make.names = TRUE: amend 'value' to correct ones:
+            value <- make.names(value, unique=TRUE)
+        ## NB: 'value' is now guaranteed to have no NA's ==> can use 'else if' :
+    }
+    else if (anyNA(value)) {
+        if(isFALSE(make.names))
+            stop("missing values in 'row.names' are not allowed")
+        if(is.na(make.names)) # automatic row.names
+            value <- .set_row_names(n)
+        else if(!isTRUE(make.names)) stop("invalid 'make.names'")
+        else # make.names = TRUE: amend 'value' to correct ones:
+            value <- make.names(value, unique=TRUE)
     }
-    if (anyNA(value))
-	stop("missing values in 'row.names' are not allowed")
     attr(x, "row.names") <- value
     x
 }
 
+`row.names<-.data.frame` <- function(x, value) `.rowNamesDF<-`(x, value=value)
+
+##_H `row.names<-.default` <- function(x, ..., value) `rownames<-`(x, value)
 `row.names<-.default` <- function(x, value) `rownames<-`(x, value)
 
 is.na.data.frame <- function (x)
@@ -141,8 +180,8 @@ as.data.frame <- function(x, row.names = NULL, optional = FALSE, ...)
 }
 
 as.data.frame.default <- function(x, ...)
-    stop(gettextf("cannot coerce class \"%s\" to a data.frame",
-                  deparse(class(x))),
+    stop(gettextf("cannot coerce class %s to a data.frame",
+                  sQuote(deparse(class(x))[1L])),
          domain = NA)
 
 ###  Here are methods ensuring that the arguments to "data.frame"
@@ -170,28 +209,30 @@ as.data.frame.data.frame <- function(x, row.names = NULL, ...)
 ## prior to 1.8.0 this coerced names - PR#3280
 as.data.frame.list <-
     function(x, row.names = NULL, optional = FALSE, ...,
+	     cut.names = FALSE, col.names = names(x), fix.empty.names = TRUE,
              stringsAsFactors = default.stringsAsFactors())
 {
     ## need to protect names in x.
-    cn <- names(x)
-    m <- match(c("row.names", "check.rows", "check.names", "stringsAsFactors"),
-               cn, 0L)
-    if(any(m)) {
-        cn[m] <- paste0("..adfl.", cn[m])
-        names(x) <- cn
-    }
-    x <- eval(as.call(c(expression(data.frame), x, check.names = !optional,
-                        stringsAsFactors = stringsAsFactors)))
-    if(any(m)) names(x) <- sub("^\\.\\.adfl\\.", "", names(x))
-    if(!is.null(row.names)) {
-	# row.names <- as.character(row.names)
-	if(length(row.names) != dim(x)[[1L]])
-            stop(sprintf(ngettext(length(row.names),
-                                  "supplied %d row name for %d rows",
-                                  "supplied %d row names for %d rows"),
-                         length(row.names), dim(x)[[1L]]), domain = NA)
-	attr(x, "row.names") <- row.names
+    ## truncate any of more than 256 (or cut.names) bytes:
+    new.nms <- !missing(col.names)
+    if(cut.names) {
+	maxL <- if(is.logical(cut.names)) 256L else as.integer(cut.names)
+	if(any(long <- nchar(col.names, "bytes", keepNA = FALSE) > maxL))
+	    col.names[long] <- paste(substr(col.names[long], 1L, maxL - 6L), "...")
+	else cut.names <- FALSE
     }
+    m <- match(names(formals(data.frame))[-1L],
+	       ## c("row.names", "check.rows", ...., "stringsAsFactors"),
+	       col.names, 0L)
+    if(any.m <- any(m)) col.names[m] <- paste0("..adfl.", col.names[m])
+    if(new.nms || any.m || cut.names) names(x) <- col.names
+    if(is.null(check.n <- list(...)$check.names)) check.n <- !optional
+    ## data.frame() is picky with its 'row.names':
+    alis <- c(list(check.names = check.n, fix.empty.names = fix.empty.names,
+		   stringsAsFactors = stringsAsFactors),
+	      if(!is.null(row.names)) list(row.names = row.names))
+    x <- do.call(data.frame, c(x, alis))
+    if(any.m) names(x) <- sub("^\\.\\.adfl\\.", "", names(x))
     x
 }
 
@@ -204,6 +245,7 @@ as.data.frame.vector <- function(x, row.names = NULL, optional = FALSE, ...,
     nrows <- length(x)
     ## ## row.names -- for now warn about and "forget" illegal row.names
     ## ##           -- can simplify much (move this *after* the is.null(.) case) once we stop() !
+### FIXME: allow  integer [of full length]
     if(!(is.null(row.names) || (is.character(row.names) && length(row.names) == nrows))) {
 	warning(gettextf(
 	    "'row.names' is not a character vector of length %d -- omitting it. Will be an error!",
@@ -236,9 +278,11 @@ as.data.frame.raw  <- as.data.frame.vector
 as.data.frame.factor  <- as.data.frame.vector
 as.data.frame.ordered <- as.data.frame.vector
 as.data.frame.integer <- as.data.frame.vector
+as.data.frame.logical <- as.data.frame.vector
 as.data.frame.numeric <- as.data.frame.vector
 as.data.frame.complex <- as.data.frame.vector
 
+
 default.stringsAsFactors <- function()
 {
     val <- getOption("stringsAsFactors")
@@ -252,27 +296,26 @@ default.stringsAsFactors <- function()
 as.data.frame.character <-
     function(x, ..., stringsAsFactors = default.stringsAsFactors())
 {
-    nm <- deparse(substitute(x), width.cutoff=500L)
+    nm <- paste(deparse(substitute(x), width.cutoff=500L), collapse = " ")# as in  as.DF.vector()
     if(stringsAsFactors) x <- factor(x)
     if(!"nm" %in% names(list(...)))
         as.data.frame.vector(x, ..., nm = nm)
     else as.data.frame.vector(x, ...)
 }
 
-as.data.frame.logical <- as.data.frame.vector
-
-as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE, ...,
+as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE, make.names = TRUE, ...,
                                  stringsAsFactors = default.stringsAsFactors())
 {
     d <- dim(x)
-    nrows <- d[1L]
-    ncols <- d[2L]
+    nrows <- d[[1L]]
+    ncols <- d[[2L]]
     ic <- seq_len(ncols)
     dn <- dimnames(x)
     ## surely it cannot be right to override the supplied row.names?
     ## changed in 1.8.0
     if(is.null(row.names)) row.names <- dn[[1L]]
     collabs <- dn[[2L]]
+    ## These might be NA
     if(any(empty <- !nzchar(collabs)))
 	collabs[empty] <- paste0("V", ic)[empty]
     value <- vector("list", ncols)
@@ -284,25 +327,29 @@ as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE, ...,
 	    value[[i]] <- as.vector(x[,i])
     }
     ## Explicitly check for NULL in case nrows==0
-    if(is.null(row.names) || length(row.names) != nrows)
-	row.names <- .set_row_names(nrows)
+    autoRN <- (is.null(row.names) || length(row.names) != nrows)
     if(length(collabs) == ncols)
 	names(value) <- collabs
     else if(!optional)
 	names(value) <- paste0("V", ic)
-    attr(value, "row.names") <- row.names
     class(value) <- "data.frame"
+    if(autoRN)
+        attr(value, "row.names") <- .set_row_names(nrows)
+    else
+        .rowNamesDF(value, make.names=make.names) <- row.names
     value
 }
 
 as.data.frame.model.matrix <-
-    function(x, row.names = NULL, optional = FALSE, ...)
+    function(x, row.names = NULL, optional = FALSE, make.names = TRUE, ...)
 {
     d <- dim(x)
-    nrows <- d[1L]
+    nrows <- d[[1L]]
     dn <- dimnames(x)
     row.names <- dn[[1L]]
     value <- list(x)
+    if(!optional) names(value) <- deparse(substitute(x))[[1L]]
+    class(value) <- "data.frame"
     if(!is.null(row.names)) {
 	row.names <- as.character(row.names)
 	if(length(row.names) != nrows)
@@ -310,11 +357,9 @@ as.data.frame.model.matrix <-
                                   "supplied %d row name for %d rows",
                                   "supplied %d row names for %d rows"),
                           length(row.names), nrows), domain = NA)
+        .rowNamesDF(value, make.names=make.names) <- row.names
     }
-    else row.names <- .set_row_names(nrows)
-    if(!optional) names(value) <- deparse(substitute(x))[[1L]]
-    attr(value, "row.names") <- row.names
-    class(value) <- "data.frame"
+    else attr(value, "row.names") <- .set_row_names(nrows)
     value
 }
 
@@ -345,6 +390,7 @@ as.data.frame.array <- function(x, row.names = NULL, optional = FALSE, ...)
 `[.AsIs` <- function(x, i, ...) I(NextMethod("["))
 
 
+## NB: this is called relatively often from data.frame() itself, ...
 as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE, ...)
 {
     if(length(dim(x)) == 2L)
@@ -353,16 +399,27 @@ as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE, ...)
         nrows <- length(x)
         nm <- paste(deparse(substitute(x), width.cutoff=500L), collapse=" ")
         if(is.null(row.names)) {
+            autoRN <- FALSE
             if (nrows == 0L)
                 row.names <- character()
             else if(length(row.names <- names(x)) == nrows &&
-                    !anyDuplicated(row.names)) {}
-            else row.names <- .set_row_names(nrows)
-        }
+                    !anyDuplicated(row.names)) {
+            }
+            else {
+                autoRN <- TRUE
+                row.names <- .set_row_names(nrows)
+            }
+        } else
+            autoRN <- is.integer(row.names) && length(row.names) == 2L &&
+                is.na(rn1 <- row.names[[1L]]) && rn1 < 0
         value <- list(x)
         if(!optional) names(value) <- nm
-        attr(value, "row.names") <- row.names
         class(value) <- "data.frame"
+        ## FIXME -- Need to comment the  'row.names(.) <-'  case
+        ## if(autoRN)
+            attr(value, "row.names") <- row.names
+        ## else
+        ##     row.names(value) <- row.names
         value
     }
 
@@ -373,6 +430,7 @@ as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE, ...)
 
 data.frame <-
     function(..., row.names = NULL, check.rows = FALSE, check.names = TRUE,
+	     fix.empty.names = TRUE,
              stringsAsFactors = default.stringsAsFactors())
 {
     data.row.names <-
@@ -386,21 +444,24 @@ data.frame <-
 		    return(new)
 		if(all(current == new) || all(current == ""))
 		    return(new)
-		stop(gettextf("mismatch of row names in arguments of 'data.frame\', item %d", i), domain = NA)
+		stop(gettextf(
+		    "mismatch of row names in arguments of 'data.frame\', item %d", i),
+		    domain = NA)
 	    }
 	else function(current, new, i) {
 	    if(is.null(current)) {
 		if(anyDuplicated(new)) {
-		    warning(gettextf("some row.names duplicated: %s --> row.names NOT used",
-                                     paste(which(duplicated(new)), collapse=",")),
-                            domain = NA)
+		    warning(gettextf(
+                        "some row.names duplicated: %s --> row.names NOT used",
+                        paste(which(duplicated(new)), collapse=",")),
+                        domain = NA)
 		    current
 		} else new
 	    } else current
 	}
     object <- as.list(substitute(list(...)))[-1L]
     mirn <- missing(row.names) # record before possibly changing
-    mrn <- is.null(row.names) # missing or NULL
+    mrn  <- is.null(row.names) # missing or NULL
     x <- list(...)
     n <- length(x)
     if(n < 1L) {
@@ -428,36 +489,32 @@ data.frame <-
     for(i in seq_len(n)) {
         ## do it this way until all as.data.frame methods have been updated
 	xi <- if(is.character(x[[i]]) || is.list(x[[i]]))
-                 as.data.frame(x[[i]], optional = TRUE,
-                               stringsAsFactors = stringsAsFactors)
-        else as.data.frame(x[[i]], optional = TRUE)
+		  as.data.frame(x[[i]], optional = TRUE,
+				stringsAsFactors = stringsAsFactors)
+	      else as.data.frame(x[[i]], optional = TRUE)
 
         nrows[i] <- .row_names_info(xi) # signed for now
 	ncols[i] <- length(xi)
 	namesi <- names(xi)
 	if(ncols[i] > 1L) {
 	    if(length(namesi) == 0L) namesi <- seq_len(ncols[i])
-	    if(no.vn[i]) vnames[[i]] <- namesi
-	    else vnames[[i]] <- paste(vnames[[i]], namesi, sep=".")
-	}
-	else {
-            if(length(namesi)) vnames[[i]] <- namesi
-            else if (no.vn[[i]]) {
-                tmpname <- deparse(object[[i]], nlines = 1L)[1L]
-                if( substr(tmpname, 1L, 2L) == "I(" ) {
-                    ntmpn <- nchar(tmpname, "c")
-                    if(substr(tmpname, ntmpn, ntmpn) == ")")
-                        tmpname <- substr(tmpname, 3L, ntmpn - 1L)
-                }
-                vnames[[i]] <- tmpname
-            }
-        } # end of ncols[i] <= 1
+	    vnames[[i]] <- if(no.vn[i]) namesi
+			   else paste(vnames[[i]], namesi, sep=".")
+	} else if(length(namesi)) {
+	    vnames[[i]] <- namesi
+	} else if (fix.empty.names && no.vn[[i]]) {
+	    tmpname <- deparse(object[[i]], nlines = 1L)[1L]
+	    if(substr(tmpname, 1L, 2L) == "I(") { ## from 'I(*)', only keep '*':
+		ntmpn <- nchar(tmpname, "c")
+		if(substr(tmpname, ntmpn, ntmpn) == ")")
+		    tmpname <- substr(tmpname, 3L, ntmpn - 1L)
+	    }
+	    vnames[[i]] <- tmpname
+	} ## else vnames[[i]] are not changed
 	if(mirn && nrows[i] > 0L) {
             rowsi <- attr(xi, "row.names")
             ## Avoid all-blank names
-            nc <- nchar(rowsi, allowNA = FALSE)
-            nc <- nc[!is.na(nc)]
-            if(length(nc) && any(nc))
+            if(any(nzchar(rowsi)))
                 row.names <- data.row.names(row.names, rowsi, i)
         }
         nrows[i] <- abs(nrows[i])
@@ -496,11 +553,16 @@ data.frame <-
     value <- unlist(vlist, recursive=FALSE, use.names=FALSE)
     ## unlist() drops i-th component if it has 0 columns
     vnames <- unlist(vnames[ncols > 0L])
-    noname <- !nzchar(vnames)
-    if(any(noname))
-	vnames[noname] <- paste("Var", seq_along(vnames), sep = ".")[noname]
-    if(check.names)
-	vnames <- make.names(vnames, unique=TRUE)
+    if(fix.empty.names && any(noname <- !nzchar(vnames)))
+	vnames[noname] <- paste0("Var.", seq_along(vnames))[noname]
+    if(check.names) {
+	if(fix.empty.names)
+	    vnames <- make.names(vnames, unique=TRUE)
+	else { ## do not fix ""
+	    nz <- nzchar(vnames)
+	    vnames[nz] <- make.names(vnames[nz], unique=TRUE)
+	}
+    }
     names(value) <- vnames
     if(!mrn) { # non-null row.names arg was supplied
         if(length(row.names) == 1L && nr != 1L) {  # one of the variables
@@ -518,8 +580,9 @@ data.frame <-
         warning("row names were found from a short variable and have been discarded")
         row.names <- NULL
     }
+    class(value) <- "data.frame"
     if(is.null(row.names))
-        row.names <- .set_row_names(nr) #seq_len(nr)
+        attr(value, "row.names") <- .set_row_names(nr) #seq_len(nr)
     else {
         if(is.object(row.names) || !is.integer(row.names))
             row.names <- as.character(row.names)
@@ -530,9 +593,8 @@ data.frame <-
                           paste(unique(row.names[duplicated(row.names)]),
                                 collapse = ", ")),
                  domain = NA)
+        row.names(value) <- row.names
     }
-    attr(value, "row.names") <- row.names
-    attr(value, "class") <- "data.frame"
     value
 }
 
@@ -784,11 +846,12 @@ data.frame <-
             has.j <- TRUE
         }
     }
-    else {
+    else # nargs() <= 2
 	stop("need 0, 1, or 2 subscripts")
-    }
-    ## no columns specified
-    if(has.j && length(j) == 0L) return(x)
+
+    if ((has.j && !length(j)) ||	# "no", i.e. empty columns specified
+        (has.i && !length(i) && !has.j))# empty rows and no col.   specified
+	return(x)
 
     cl <- oldClass(x)
     ## delete class: S3 idiom to avoid any special methods for [[, etc
@@ -810,7 +873,8 @@ data.frame <-
 	    }
 	    i <- ii
 	}
-	if(all(i >= 0L) && (nn <- max(i)) > nrows) {
+	if(!is.logical(i) &&
+	   (char.i && nextra  ||  all(i >= 0L) && (nn <- max(i)) > nrows)) {
 	    ## expand
             if(is.null(rows)) rows <- attr(x, "row.names")
 	    if(!char.i) {
@@ -837,14 +901,12 @@ data.frame <-
             stop("missing values are not allowed in subscripted assignments of data frames")
 	if(is.character(j)) {
             if("" %in% j) stop("column name \"\" cannot match any column")
-	    jj <- match(j, names(x))
-	    nnew <- sum(is.na(jj))
-	    if(nnew > 0L) {
-		n <- is.na(jj)
-		jj[n] <- nvars + seq_len(nnew)
+	    jseq <- match(j, names(x))
+	    if(anyNA(jseq)) {
+		n <- is.na(jseq)
+		jseq[n] <- nvars + seq_len(sum(n))
 		new.cols <- j[n]
 	    }
-	    jseq <- jj
 	}
 	else if(is.logical(j) || min(j) < 0L)
 	    jseq <- seq_along(x)[j]
@@ -866,12 +928,19 @@ data.frame <-
     }
     else jseq <- seq_along(x)
 
+    ## empty rows and not (a *new* column as in  d[FALSE, "new"] <- val )  :
+    if(has.i && !length(iseq) && all(1L <= jseq & jseq <= nvars))
+	return(`class<-`(x, cl))
+
     ## addition in 1.8.0
     if(anyDuplicated(jseq))
         stop("duplicate subscripts for columns")
     n <- length(iseq)
     if(n == 0L) n <- nrows
     p <- length(jseq)
+    if (is.null(value)) {
+        value <- list(NULL)
+    }
     m <- length(value)
     if(!is.list(value)) {
         if(p == 1L) {
@@ -950,7 +1019,7 @@ data.frame <-
     ncolv <- dimv[2L]
     jvseq <- seq_len(p)
     if(ncolv < p) jvseq <- rep_len(seq_len(ncolv), p)
-    else if(ncolv > p) {
+    else if(p != 0L && ncolv > p) {
         warning(sprintf(ngettext(ncolv,
                                  "provided %d variable to replace %d variables",
                                  "provided %d variables to replace %d variables"),
@@ -973,17 +1042,18 @@ data.frame <-
 	    vjj <- value[[ jvseq[[jjj]] ]]
             if(jj <= nvars) {
                 ## if a column exists, preserve its attributes
-                if(length(dim(x[[jj]])) != 2L) x[[jj]][iseq] <- vjj
+                if(length(dim(x[[jj]])) != 2L)
+                     x[[jj]][iseq  ] <- vjj
                 else x[[jj]][iseq, ] <- vjj
             } else {
                 ## try to make a new column match in length: may be an error
                 x[[jj]] <- vjj[FALSE]
                 if(length(dim(vjj)) == 2L) {
-                    length(x[[j]]) <- nrows * ncol(vjj)
-                    dim(x[[j]]) <- c(nrows, ncol(vjj))
+                    length(x[[jj]]) <- nrows * ncol(vjj)
+                    dim(x[[jj]])  <- c(nrows,  ncol(vjj))
                     x[[jj]][iseq, ] <- vjj
                 } else {
-                    length(x[[j]]) <- nrows
+                    length(x[[jj]]) <- nrows
                     x[[jj]][iseq] <- vjj
                 }
             }
@@ -999,7 +1069,7 @@ data.frame <-
         v <- value[[ jvseq[[jjj]] ]]
         ## This is consistent with the have.i case rather than with
         ## [[<- and $<- (which throw an error).  But both are plausible.
-        if (nrows > 0L && !length(v)) length(v) <- nrows
+        if (!is.null(v) && nrows > 0L && !length(v)) length(v) <- nrows
 	x[[jj]] <- v
         if (!is.null(v) && is.atomic(x[[jj]]) && !is.null(names(x[[jj]])))
             names(x[[jj]]) <- NULL
@@ -1142,20 +1212,6 @@ data.frame <-
     return(x)
 }
 
-### Added for 3.1.0
-`$.data.frame` <- function(x,name) {
-  a <- x[[name]]
-  if (!is.null(a)) return(a)
-
-  a <- x[[name, exact=FALSE]]
-  if (!is.null(a) && getOption("warnPartialMatchDollar", default=FALSE)) {
-  	names <- names(x)
-  	warning(gettextf("Partial match of '%s' to '%s' in data frame",
-                                   name, names[pmatch(name, names)]))
-  }
-  return(a)
-}
-
 
 xpdrows.data.frame <- function(x, old.rows, new.rows)
 {
@@ -1187,7 +1243,9 @@ xpdrows.data.frame <- function(x, old.rows, new.rows)
 	    x[[i]] <- y
 	}
     }
-    attr(x, "row.names") <- c(old.rows, new.rows)
+    nm <- c(old.rows, new.rows)
+    if (any(duplicated(nm))) nm <- make.unique(as.character(nm))
+    attr(x, "row.names") <- nm
     x
 }
 
@@ -1211,26 +1269,15 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
             m
 	} else stop("names do not match previous names")
     }
-    if(make.row.names)
-    Make.row.names <- function(nmi, ri, ni, nrow)
-    {
-	if(nzchar(nmi)) {
-            if(ni == 0L) character()  # PR8506
-	    else if(ni > 1L) paste(nmi, ri, sep = ".")
-	    else nmi
-	}
-	else if(nrow > 0L && identical(ri, seq_len(ni)))
-	    as.integer(seq.int(from = nrow + 1L, length.out = ni))
-	else ri
-    }
     allargs <- list(...)
     allargs <- allargs[lengths(allargs) > 0L]
     if(length(allargs)) {
-    ## drop any zero-row data frames, as they may not have proper column
-    ## types (e.g. NULL).
+        ## drop any zero-row data frames, as they may not have proper column
+        ## types (e.g. NULL).
         nr <- vapply(allargs, function(x)
                      if(is.data.frame(x)) .row_names_info(x, 2L)
-                     else if(is.list(x)) length(x[[1L]]) # mismatched lists are checked later
+                     else if(is.list(x)) length(x[[1L]])
+					# mismatched lists are checked later
                      else length(x), 1L)
         if(any(nr > 0L)) allargs <- allargs[nr > 0L]
         else return(allargs[[1L]]) # pretty arbitrary
@@ -1245,7 +1292,26 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
 	nms <- character(n)
     cl <- NULL
     perm <- rows <- vector("list", n)
-    rlabs <- if(make.row.names) rows # else NULL
+    if(make.row.names) {
+	rlabs <- rows
+	autoRnms <- TRUE # result with 1:nrow(.) row names? [efficiency!]
+	Make.row.names <- function(nmi, ri, ni, nrow)
+	{
+	    if(nzchar(nmi)) {
+		if(autoRnms) autoRnms <<- FALSE
+		if(ni == 0L) character()  # PR#8506
+		else if(ni > 1L) paste(nmi, ri, sep = ".")
+		else nmi
+	    }
+	    else if(autoRnms && nrow > 0L && identical(ri, seq_len(ni)))
+		as.integer(seq.int(from = nrow + 1L, length.out = ni))
+	    else {
+		if(autoRnms && (nrow > 0L || !identical(ri, seq_len(ni))))
+		    autoRnms <<- FALSE
+		ri
+	    }
+	}
+    }
     nrow <- 0L
     value <- clabs <- NULL
     all.levs <- list()
@@ -1279,10 +1345,12 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
 		has.dim <- facCol <- ordCol <- logical(nvar)
 		for(j in seq_len(nvar)) {
 		    xj <- value[[j]]
-		    if( !is.null(levels(xj)) ) {
-			all.levs[[j]] <- levels(xj)
-                        facCol[j] <- TRUE # turn categories into factors
-                    } else facCol[j] <- is.factor(xj)
+                    facCol[j] <-
+                        if(!is.null(levels(xj))) {
+                            all.levs[[j]] <- levels(xj)
+                            TRUE # turn categories into factors
+                        } else
+                            is.factor(xj)
                     ordCol[j] <- is.ordered(xj)
 		    has.dim[j] <- length(dim(xj)) == 2L
 		}
@@ -1319,9 +1387,10 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
 		}
 	    }
 	}
-	else if(length(xi)) {
+	else if(length(xi)) { # 1 new row
 	    rows[[i]] <- nrow <- nrow + 1L
-            if(make.row.names) rlabs[[i]] <- if(nzchar(nmi)) nmi else as.integer(nrow)
+            if(make.row.names)
+		rlabs[[i]] <- if(nzchar(nmi)) nmi else as.integer(nrow)
 	}
     }
     nvar <- length(clabs)
@@ -1378,13 +1447,15 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
             }
 	}
     }
-    if(make.row.names) {
-	rlabs <- unlist(rlabs)
-	if(anyDuplicated(rlabs))
-	    rlabs <- make.unique(as.character(rlabs), sep = "")
-    }
+    rlabs <- if(make.row.names && !autoRnms) {
+		 rlabs <- unlist(rlabs)
+		 if(anyDuplicated(rlabs))
+		     make.unique(as.character(rlabs), sep = "")
+		 else
+		     rlabs
+	     } # else NULL
     if(is.null(cl)) {
-	as.data.frame(value, row.names = rlabs,
+	as.data.frame(value, row.names = rlabs, fix.empty.names = TRUE,
 		      stringsAsFactors = stringsAsFactors)
     } else {
 	structure(value, class = cl,
@@ -1397,24 +1468,33 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
 
 print.data.frame <-
     function(x, ..., digits = NULL, quote = FALSE, right = TRUE,
-	     row.names = TRUE)
+	     row.names = TRUE, max = NULL)
 {
     n <- length(row.names(x))
     if(length(x) == 0L) {
-        cat(sprintf(ngettext(n, "data frame with 0 columns and %d row",
-                             "data frame with 0 columns and %d rows",
-                             domain = "R-base"), n), "\n", sep = "")
+	cat(sprintf(ngettext(n, "data frame with 0 columns and %d row",
+			     "data frame with 0 columns and %d rows"),
+		    n), "\n", sep = "")
     } else if(n == 0L) {
         ## FIXME: header format is inconsistent here
 	print.default(names(x), quote = FALSE)
 	cat(gettext("<0 rows> (or 0-length row.names)\n"))
     } else {
+	if(is.null(max)) max <- getOption("max.print", 99999L)
+        if(!is.finite(max)) stop("invalid 'max' / getOption(\"max.print\"): ", max)
 	## format.<*>() : avoiding picking up e.g. format.AsIs
-	m <- as.matrix(format.data.frame(x, digits = digits, na.encode = FALSE))
+	omit <- (n0 <- max %/% length(x)) < n
+	m <- as.matrix(
+	    format.data.frame(if(omit) x[seq_len(n0), , drop=FALSE] else x,
+			      digits = digits, na.encode = FALSE))
 	if(!isTRUE(row.names))
-	    dimnames(m)[[1L]] <- if(identical(row.names,FALSE))
-		rep.int("", n) else row.names
-	print(m, ..., quote = quote, right = right)
+	    dimnames(m)[[1L]] <-
+		if(isFALSE(row.names)) rep.int("", if(omit) n0 else n)
+		else row.names
+	print(m, ..., quote = quote, right = right, max = max)
+	if(omit)
+	    cat(" [ reached 'max' / getOption(\"max.print\") -- omitted",
+		n - n0, "rows ]\n")
     }
     invisible(x)
 }
@@ -1423,30 +1503,29 @@ as.matrix.data.frame <- function (x, rownames.force = NA, ...)
 {
     dm <- dim(x)
     rn <- if(rownames.force %in% FALSE) NULL
-    else if(rownames.force %in% TRUE) row.names(x)
-    else {if(.row_names_info(x) <= 0L) NULL else row.names(x)}
+	  else if(rownames.force %in% TRUE || .row_names_info(x) > 0L)
+              row.names(x) # else NULL
     dn <- list(rn, names(x))
     if(any(dm == 0L))
 	return(array(NA, dim = dm, dimnames = dn))
-    p <- dm[2L]
+    p <- dm[2L] # >= 1
     pseq <- seq_len(p)
     n <- dm[1L]
-    X <- x # will contain the result;
+    X <- unclass(x) # will contain the result;
     ## the "big question" is if we return a numeric or a character matrix
-    class(X) <- NULL
     non.numeric <- non.atomic <- FALSE
     all.logical <- TRUE
     for (j in pseq) {
-        if(inherits(X[[j]], "data.frame") && ncol(xj) > 1L)
-            X[[j]] <- as.matrix(X[[j]])
-        xj <- X[[j]]
+	xj <- X[[j]]
+	if(inherits(xj, "data.frame"))# && ncol(xj) > 1L)
+	    X[[j]] <- xj <- as.matrix(xj)
         j.logic <- is.logical(xj)
         if(all.logical && !j.logic) all.logical <- FALSE
 	if(length(levels(xj)) > 0L || !(j.logic || is.numeric(xj) || is.complex(xj))
 	   || (!is.null(cl <- attr(xj, "class")) && # numeric classed objects to format:
 	       any(cl %in% c("Date", "POSIXct", "POSIXlt"))))
 	    non.numeric <- TRUE
-	if(!is.atomic(xj))
+	if(!is.atomic(xj) && !inherits(xj, "POSIXlt"))
 	    non.atomic <- TRUE
     }
     if(non.atomic) {
@@ -1461,10 +1540,13 @@ as.matrix.data.frame <- function (x, rownames.force = NA, ...)
 	for (j in pseq) {
 	    if (is.character(X[[j]]))
 		next
-	    xj <- X[[j]]
-            miss <- is.na(xj)
-	    xj <- if(length(levels(xj))) as.vector(xj) else format(xj)
-            is.na(xj) <- miss
+	    else if(is.logical(xj <- X[[j]]))
+		xj <- as.character(xj) # not format(), takes care of NAs too
+	    else {
+		miss <- is.na(xj)
+		xj <- if(length(levels(xj))) as.vector(xj) else format(xj)
+		is.na(xj) <- miss
+	    }
             X[[j]] <- xj
 	}
     }
@@ -1475,18 +1557,22 @@ as.matrix.data.frame <- function (x, rownames.force = NA, ...)
     for (j in pseq) {
         xj <- X[[j]]
         dj <- dim(xj)
-	if(length(dj) == 2L && dj[2L] > 1L) { # matrix with >=2 col
-	    dnj <- colnames(xj)
-	    collabs[[j]] <- paste(collabs[[j]],
-				  if(length(dnj)) dnj else seq_len(dj[2L]),
-				  sep = ".")
-	}
-     }
+        if(length(dj) == 2L && dj[2L] > 0L) { # matrix with > 0 col
+            if(!length(dnj <- colnames(xj))) dnj <- seq_len(dj[2L])
+            collabs[[j]] <-
+                if(length(collabs)) {
+                    if(dj[2L] > 1L)
+                        paste(collabs[[j]], dnj, sep = ".")
+                    else if(is.character(collabs[[j]])) collabs[[j]]
+                    else dnj
+                }
+                else dnj
+        }
+    }
+    nc <- vapply(X, NCOL, numeric(1), USE.NAMES=FALSE)
     X <- unlist(X, recursive = FALSE, use.names = FALSE)
     dim(X) <- c(n, length(X)/n)
-    dimnames(X) <- list(dn[[1L]], unlist(collabs, use.names = FALSE))
-    ##NO! don't copy buggy S-plus!  either all matrices have class or none!!
-    ##NO class(X) <- "matrix"
+    dimnames(X) <- list(dn[[1L]], unlist(collabs[nc > 0], use.names = FALSE))
     X
 }
 
@@ -1499,7 +1585,8 @@ Math.data.frame <- function (x, ...)
     } else {
 	vnames <- names(x)
 	if (is.null(vnames)) vnames <- seq_along(x)
-	stop("non-numeric variable in data frame: ", vnames[!mode.ok])
+	stop("non-numeric variable(s) in data frame: ",
+	     paste(vnames[!mode.ok], collapse = ", "))
     }
 }
 
@@ -1560,17 +1647,22 @@ Ops.data.frame <- function(e1, e2 = NULL)
 	}
     }
     for(j in seq_along(cn)) {
-	left <- if(!lscalar) e1[[j]] else e1
+	left  <- if(!lscalar) e1[[j]] else e1
 	right <- if(!rscalar) e2[[j]] else e2
 	value[[j]] <- eval(f)
     }
-    if(.Generic %in% c("+","-","*","/","%%","%/%") ) {
-	names(value) <- cn
-	data.frame(value, row.names = rn, check.names = FALSE,
-                   check.rows = FALSE)
+    if(.Generic %in% c("+","-","*","^","%%","%/%","/")) {## == 'Arith'
+	if(length(value)) {
+	    names(value) <- cn
+	    data.frame(value, row.names = rn, check.names = FALSE)
+	} else
+	    data.frame(       row.names = rn, check.names = FALSE)
+    }
+    else { ## 'Logic' ("&","|")  and  'Compare' ("==",">","<","!=","<=",">=") :
+	value <- unlist(value, recursive = FALSE, use.names = FALSE)
+	matrix(if(is.null(value)) logical() else value,
+	       nrow = nr, dimnames = list(rn,cn))
     }
-    else matrix(unlist(value, recursive = FALSE, use.names = FALSE),
-		nrow = nr, dimnames = list(rn,cn))
 }
 
 Summary.data.frame <- function(..., na.rm)
