@@ -1,7 +1,7 @@
 #  File src/library/methods/R/BasicClasses.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -38,17 +38,19 @@
     clList <- c("VIRTUAL", "ANY", "vector", "missing")
     for(.class in clList)
         setClass(.class, where = envir)
+    ## Now some pseudo-classes in base, marked specially for new()
     ## "numeric" is the class returned by class() for double vectors
     vClasses <- c("logical", "numeric", "character",
+                  "double",
                   "complex", "integer", "raw",
                   "expression", "list")
-    ## now some pseudo-classes in base, marked specially for new()
     for(.class in vClasses) {
         .setBaseClass(.class, prototype = newBasic(.class), where = envir)
     }
     .setBaseClass("expression", prototype = expression(), where = envir)
     clList <- c(clList, vClasses)
     nullF <- function()NULL; environment(nullF) <- .GlobalEnv
+    nullF <- utils::removeSource(nullF)
     attr(nullF, "source") <- NULL
     .setBaseClass("function", prototype = nullF, where = envir); clList <- c(clList, "function")
 
@@ -95,8 +97,12 @@
     for(.class in vClasses)
         setIs(.class, "vector", where = envir)
 
+    ## The one place where "double" and "numeric" currently differ:
+    setIs("integer", "double", where = envir,
+          coerce  = .gblEnv(function(object) as.double(object)),
+          replace = .gblEnv(function(from, value) { class(value) <- "integer" ; value }))
     setIs("integer", "numeric", where = envir)
-
+    setIs("double",  "numeric", where = envir)
     setIs("structure", "vector", coerce = .gblEnv(function(object) as.vector(object)),
           replace = .gblEnv(function(from, to, value) {
               attributes(value) <- attributes(from)
@@ -201,17 +207,10 @@
               },
               where = envir)
     setClass("factor", contains = "integer", representation(levels = "character"),
-	     validity = function(object) {
-		 levs <- levels(object)
-		 if (!is.character(levs))
-		     return("factor levels must be \"character\"")
-		 if (d <- anyDuplicated(levs))
-		     return(sprintf("duplicated level [%d] in factor", d))
-		 ## 'else'	ok :
-		 TRUE
-	     },
-	     where = envir)
+	     validity = base::.valid.factor, where = envir)
     setOldClass("factor", S4Class = "factor", where = envir)
+    setClass("ordered", contains = "factor", where = envir)
+    setOldClass("ordered", S4Class = "ordered", where = envir)
     if(!isGeneric("show", envir))
         setGeneric("show", where = envir, simpleInheritanceOnly = TRUE)
     setMethod("show", "oldClass", function(object) {
@@ -363,7 +362,7 @@
                   else {
                       cl <- .class1(from)
                       classDef <- getClass(cl)
-                      if(identical(classDef@virtual, TRUE))
+                      if(isTRUE(classDef@virtual))
                         stop(gettextf("class %s is VIRTUAL; not meaningful to create an S4 object from this class",
                                       dQuote(cl)),
                              domain = NA)
@@ -535,7 +534,6 @@
   list(
        list("data.frame",  data.frame(), "data.frame"),
        list("factor",  factor()),
-       list(c("ordered", "factor"), ordered(character())),
        list("table",  table(factor())),
        list("summary.table",  summary.table(table(factor())))
        , list("ts", stats::ts())
