@@ -498,7 +498,7 @@ function(file, pdf = FALSE, clean = FALSE, quiet = TRUE,
 ### ** .BioC_version_associated_with_R_version
 
 .BioC_version_associated_with_R_version <-
-    function() numeric_version(Sys.getenv("R_BIOC_VERSION", "3.8"))
+    function() numeric_version(Sys.getenv("R_BIOC_VERSION", "3.9"))
 ## Things are more complicated from R-2.15.x with still two BioC
 ## releases a year, so we do need to set this manually.
 ## Wierdly, 3.0 is the second version (after 2.14) for the 3.1.x series.
@@ -538,8 +538,9 @@ function(file, pdf = FALSE, clean = FALSE, quiet = TRUE,
 
 ### ** filtergrep
 
-filtergrep <- function(pattern, x, ...) grep(pattern, x, invert = TRUE, value = TRUE, ...)
-
+filtergrep <-
+function(pattern, x, ...)
+    grep(pattern, x, invert = TRUE, value = TRUE, ...)
 
 ### ** %notin%
 
@@ -563,7 +564,14 @@ function()
     if(nzchar(OS)) OS else .Platform$OS.type
 }
 
-### .R_top_srcdir
+### ** .R_copyright_msg
+
+.R_copyright_msg <-
+function(year)
+    sprintf("Copyright (C) %s-%s The R Core Team.",
+            year, R.version$year)
+
+### ** .R_top_srcdir
 
 ## Find the root directory of the source tree used for building this
 ## version of R (corresponding to Unix configure @top_srcdir@).
@@ -603,7 +611,7 @@ function(val) {
 .canonicalize_doi <-
 function(x)
 {
-    x <- sub("^((doi|DOI):)?[[:space:]]*http://(dx[.])?doi[.]org/", "",
+    x <- sub("^((doi|DOI):)?[[:space:]]*https?://(dx[.])?doi[.]org/", "",
              x)
     sub("^(doi|DOI):", "", x)
 }
@@ -613,10 +621,13 @@ function(x)
 .canonicalize_quotes <-
 function(txt)
 {
+    txt <- as.character(txt)
+    enc <- Encoding(txt)
     txt <- gsub(paste0("(", intToUtf8(0x2018), "|", intToUtf8(0x2019), ")"),
                 "'", txt, perl = TRUE, useBytes = TRUE)
     txt <- gsub(paste0("(", intToUtf8(0x201c), "|", intToUtf8(0x201d), ")"),
-                "'", txt, perl = TRUE, useBytes = TRUE)
+                '"', txt, perl = TRUE, useBytes = TRUE)
+    Encoding(txt) <- enc
     txt
 }
 
@@ -707,7 +718,7 @@ function(file1, file2)
 .file_path_relative_to_dir <-
 function(x, dir, add = FALSE)
 {
-    if(any(ind <- (substring(x, 1L, nchar(dir)) == dir))) {
+    if(any(ind <- startsWith(x, dir))) {
         ## Assume .Platform$file.sep is a single character.
         x[ind] <- if(add)
             file.path(basename(dir), substring(x[ind], nchar(dir) + 2L))
@@ -801,7 +812,7 @@ function(con)
     ## How can we find out for sure that there were errors?  Try
     ## guessing ... and peeking at tex-buf.el from AUCTeX.
     really_has_errors <-
-        (length(grep("^---", lines)) ||
+        (any(startsWith(lines, "---")) ||
          regexpr("There (was|were) ([0123456789]+) error messages?",
                  lines[length(lines)]) > -1L)
     ## (Note that warnings are ignored for now.)
@@ -886,14 +897,56 @@ function(nsInfo)
     ## Get the registered S3 methods for an 'nsInfo' object returned by
     ## parseNamespaceFile(), as a 3-column character matrix with the
     ## names of the generic, class and method (as a function).
-    S3_methods_list <- nsInfo$S3methods
-    if(!length(S3_methods_list)) return(matrix(character(), ncol = 3L))
-    idx <- is.na(S3_methods_list[, 3L])
-    S3_methods_list[idx, 3L] <-
-        paste(S3_methods_list[idx, 1L],
-              S3_methods_list[idx, 2L],
+    S3_methods_db <- nsInfo$S3methods
+    if(!length(S3_methods_db))
+        return(matrix(character(), ncol = 3L))
+    idx <- is.na(S3_methods_db[, 3L])
+    S3_methods_db[idx, 3L] <-
+        paste(S3_methods_db[idx, 1L],
+              S3_methods_db[idx, 2L],
               sep = ".")
-    S3_methods_list
+    S3_methods_db
+}
+
+### ** .get_namespace_S3_methods_with_homes
+
+.get_namespace_S3_methods_with_homes <-
+function(package, lib.loc = NULL)
+{
+    ## Get the registered S3 methods with the 'homes' of the generics
+    ## they are registered for.
+    ## Original code provided by Luke Tierney.
+    path <- system.file(package = package, lib.loc = lib.loc)
+    if(!nzchar(path)) return(NULL)
+    if(package == "base") {
+        return(data.frame(generic = .S3_methods_table[, 1L],
+                          home = rep_len("base",
+                                         nrow(.S3_methods_table)),
+                          class = .S3_methods_table[, 2L],
+                          stringsAsFactors = FALSE))
+    }
+    lib.loc <- dirname(path)
+    nsinfo <- parseNamespaceFile(package, lib.loc)
+    S3methods <- nsinfo$S3methods
+    if(!length(S3methods)) return(NULL)
+    generic <- S3methods[, 1L]
+    nsenv <- loadNamespace(package, lib.loc)
+    ## Possibly speed things up by only looking up the unique generics.
+    generics <- unique(generic)
+    homes <- character(length(generics))
+    ind <- is.na(match(generics, .get_S3_group_generics()))
+    homes[ind] <-
+        unlist(lapply(generics[ind],
+                      function(f) {
+                          f <- get(f, nsenv)
+                          getNamespaceName(topenv(environment(f)))
+                      }),
+               use.names = FALSE)
+    ## S3 group generics belong to base.
+    homes[!ind] <- "base"
+    home <- homes[match(generic, generics)]
+    class <- S3methods[, 2L]
+    data.frame(generic, home, class, stringsAsFactors = FALSE)
 }
 
 ### ** .get_package_metadata
@@ -1011,15 +1064,21 @@ function(dir, installed = TRUE, primitive = FALSE)
     ## some BioC packages warn here
     suppressWarnings(
     unique(c(.get_internal_S3_generics(primitive),
-             unlist(lapply(env_list,
-                           function(env) {
-                               nms <- sort(names(env))
-                               if(".no_S3_generics" %in% nms)
-                                   character()
-                               else Filter(function(f)
-                                           .is_S3_generic(f, envir = env),
-                                           nms)
-                           })))))
+             unlist(lapply(env_list, .get_S3_generics_in_env))))
+    )
+}
+
+### ** .get_S3_generics_in_env
+
+.get_S3_generics_in_env <-
+function(env, nms = NULL)
+{
+    if(is.null(nms))
+        nms <- sort(names(env))
+    if(".no_S3_generics" %in% nms)
+        character()
+    else
+        Filter(function(f) .is_S3_generic(f, envir = env), nms)
 }
 
 ### ** .get_S3_group_generics
@@ -1072,7 +1131,27 @@ local({
     names(out) <-
         tolower(sub("^R_PKGS_([[:upper:]]+) *=.*", "\\1", lines))
     eval(substitute(function() {out}, list(out=out)), envir=NULL)
-})
+    })
+
+### ** .get_standard_package_dependencies
+
+.get_standard_package_dependencies <-
+function(reverse = FALSE, recursive = FALSE)
+{
+    names <- unlist(.get_standard_package_names())
+    paths <- file.path(.Library, names, "DESCRIPTION")
+    ## Be nice ...
+    paths <- paths[file.exists(paths)]
+    which <- c("Depends", "Imports")
+    fields <- c("Package", which)
+    ## Create a minimal available packages db.
+    a <- do.call(rbind,
+                 lapply(paths,
+                        function(p) .read_description(p)[fields]))
+    colnames(a) <- fields
+    package_dependencies(names, a, which = which,
+                         reverse = reverse, recursive = recursive)
+}
 
 ### ** .get_standard_repository_URLs
 
@@ -1165,6 +1244,7 @@ function()
                "Priority",
                "RdMacros",
                "Suggests",
+               "StagedInstall",
                "SysDataCompression",
                "SystemRequirements",
                "Title",
@@ -1285,8 +1365,9 @@ function(x)
 {
     ## Determine whether the strings in a character vector are ASCII or
     ## not.
-    vapply(as.character(x), function(txt)
-           all(charToRaw(txt) <= as.raw(127)), NA)
+    vapply(as.character(x),
+           function(txt) all(charToRaw(txt) <= as.raw(127)),
+           NA)
 }
 
 ### ** .is_ISO_8859
@@ -1298,10 +1379,12 @@ function(x)
     ## some ISO 8859 character set or not.
     raw_ub <- as.raw(0x7f)
     raw_lb <- as.raw(0xa0)
-    vapply(as.character(x), function(txt) {
-        raw <- charToRaw(txt)
-        all(raw <= raw_ub | raw >= raw_lb)
-    }, NA)
+    vapply(as.character(x),
+           function(txt) {
+               raw <- charToRaw(txt)
+               all(raw <= raw_ub | raw >= raw_lb)
+           },
+           NA)
 }
 
 ### ** .is_primitive_in_base
@@ -1532,6 +1615,8 @@ nonS3methods <- function(package)
              ##        they should not need to be repeated here:
              Matrix = c("qr.Q", "qr.R", "qr.coef", "qr.fitted",
                         "qr.qty", "qr.qy", "qr.resid"),
+             PerformanceAnalytics = c("mean.LCL", "mean.UCL",
+                                      "mean.geometric", "mean.stderr"),
              RCurl = "merge.list",
              RNetCDF = c("close.nc", "dim.def.nc", "dim.inq.nc",
                          "dim.rename.nc", "open.nc", "print.nc"),
@@ -1648,7 +1733,8 @@ function()
 ### ** .package_apply
 
 .package_apply <-
-function(packages = NULL, FUN, ...)
+function(packages = NULL, FUN, ...,
+         Ncpus = getOption("Ncpus", 1L))
 {
     ## Apply FUN and extra '...' args to all given packages.
     ## The default corresponds to all installed packages with high
@@ -1656,12 +1742,28 @@ function(packages = NULL, FUN, ...)
     if(is.null(packages))
         packages <-
             unique(utils::installed.packages(priority = "high")[ , 1L])
-    out <- lapply(packages, function(p)
-                  tryCatch(FUN(p, ...),
-                           error = function(e)
-                           noquote(paste("Error:",
-                                         conditionMessage(e)))))
+
+    one <- function(p)
+        tryCatch(FUN(p, ...),
+                 error = function(e)
+                     noquote(paste("Error:",
+                                   conditionMessage(e))))
     ## (Just don't throw the error ...)
+
+    ## Would be good to have a common wrapper ...
+    if(Ncpus > 1L) {
+        if(.Platform$OS.type != "windows") {
+            out <- parallel::mclapply(packages, one, mc.cores = Ncpus)
+        } else {
+            cl <- parallel::makeCluster(Ncpus)
+            args <- list(FUN, ...)      # Eval promises.
+            out <- parallel::parLapply(cl, packages, one)
+            parallel::stopCluster(cl)
+        }
+    } else {
+        out <- lapply(packages, one)
+    }
+
     names(out) <- packages
     out
 }
@@ -1712,7 +1814,7 @@ function(con)
     ## Read lines from a connection to an Rd file, trying to suppress
     ## "incomplete final line found by readLines" warnings.
     if(is.character(con)) {
-        con <- if(length(grep("\\.gz$", con))) gzfile(con, "r") else file(con, "r")
+        con <- if(endsWith(con, ".gz")) gzfile(con, "r") else file(con, "r")
         on.exit(close(con))
     }
     .try_quietly(readLines(con, warn=FALSE))
@@ -1828,14 +1930,20 @@ function(x, dfile)
             x[ind] <- iconv(x[ind], "latin1", "ASCII", sub = "byte")
         }
     }
-    ## Avoid declared encodings when writing out.
-    Encoding(x) <- "unknown"
+    ## <FIXME>
+    ## ## Avoid declared encodings when writing out.
+    ##   Encoding(x) <- "unknown"
+    ## This may also yield byte subs when formatting non-keep-white
+    ## fields via strwrap() (PR#17550).
+    ## Remove eventually ...
+    ## </FIXME>
     ## Avoid folding for fields where we keep whitespace when reading,
     ## plus two where legacy code does not strip whitespace and so
     ## we should not wrap the field.
     write.dcf(rbind(x), dfile,
               keep.white = c(.keep_white_description_fields,
-                             "Maintainer", "BugReports"))
+                             "Maintainer", "BugReports"),
+              useBytes = TRUE)
 }
 
 ### ** .read_repositories
@@ -2001,31 +2109,6 @@ function(x)
     } else list(name = x1)
 }
 
-## <FIXME>
-## We now have base::trimws(), so this is no longer needed.
-## Remove eventually.
-
-### ** .strip_whitespace
-
-## <NOTE>
-## Other languages have this as strtrim() (or variants for left or right
-## trimming only), but R has a different strtrim().
-## So perhaps strstrip()?
-## Could more generally do
-##   strstrip(x, pattern, which = c("both", "left", "right"))
-## </NOTE>
-
-.strip_whitespace <-
-function(x)
-{
-    ## Strip leading and trailing whitespace.
-    x <- sub("^[[:space:]]+", "", x)
-    x <- sub("[[:space:]]+$", "", x)
-    x
-}
-
-## </FIXME>
-
 ### ** .system_with_capture
 
 .system_with_capture <-
@@ -2089,21 +2172,36 @@ function(expr)
 ### ** .unpacked_source_repository_apply
 
 .unpacked_source_repository_apply <-
-function(dir, fun, ..., pattern = "*", verbose = FALSE)
+function(dir, FUN, ..., pattern = "*", verbose = FALSE,
+         Ncpus = getOption("Ncpus", 1L))
 {
     dir <- file_path_as_absolute(dir)
 
     dfiles <- Sys.glob(file.path(dir, pattern, "DESCRIPTION"))
+    paths <- dirname(dfiles)
 
-    results <-
-        lapply(dirname(dfiles),
-               function(dir) {
-                   if(verbose)
-                       message(sprintf("processing %s", basename(dir)))
-                   fun(dir, ...)
-               })
-    names(results) <- basename(dirname(dfiles))
-    results
+    one <- function(p) {
+        if(verbose)
+            message(sprintf("processing %s", basename(p)))
+        FUN(p, ...)
+    }
+
+    ## Would be good to have a common wrapper ...
+    if(Ncpus > 1L) {
+        if(.Platform$OS.type != "windows") {
+            out <- parallel::mclapply(paths, one, mc.cores = Ncpus)
+        } else {
+            cl <- parallel::makeCluster(Ncpus)
+            args <- list(FUN, ...)      # Eval promises.
+            out <- parallel::parLapply(cl, paths, one)
+            parallel::stopCluster(cl)
+        }
+    } else {
+        out <- lapply(paths, one)
+    }
+
+    names(out) <- basename(paths)
+    out
 }
 
 ### ** .wrong_args
@@ -2129,7 +2227,8 @@ function(args, msg)
 
 ### ** Rcmd
 
-Rcmd <- function(args, ...)
+Rcmd <-
+function(args, ...)
 {
     if(.Platform$OS.type == "windows")
         system2(file.path(R.home("bin"), "Rcmd.exe"), args, ...)
@@ -2139,12 +2238,14 @@ Rcmd <- function(args, ...)
 
 ### ** pskill
 
-pskill <- function(pid, signal = SIGTERM)
+pskill <-
+function(pid, signal = SIGTERM)
     invisible(.Call(C_ps_kill, pid, signal))
 
 ### ** psnice
 
-psnice <- function(pid = Sys.getpid(), value = NA_integer_)
+psnice <-
+function(pid = Sys.getpid(), value = NA_integer_)
 {
     res <- .Call(C_ps_priority, pid, value)
     if(is.na(value)) res else invisible(res)
@@ -2154,7 +2255,8 @@ psnice <- function(pid = Sys.getpid(), value = NA_integer_)
 
 ## original version based on http://daringfireball.net/2008/05/title_case
 ## but much altered before release.
-toTitleCase <- function(text)
+toTitleCase <-
+function(text)
 {
     ## leave these alone: the internal caps rule would do that
     ## in some cases.  We could insist on this exact capitalization.
@@ -2208,7 +2310,8 @@ toTitleCase <- function(text)
 ### ** path_and_libPath
 
 ##' Typically the union of R_LIBS and current .libPaths(); may differ e.g. via R_PROFILE
-path_and_libPath <- function(...)
+path_and_libPath <-
+function(...)
 {
     lP <- .libPaths()
     ## don't call normalizePath on paths which do not exist: allowed in R_LIBS!
@@ -2220,7 +2323,9 @@ path_and_libPath <- function(...)
 ### ** str_parse_logic
 
 ##' @param otherwise: can be call, such as quote(errmesg(...))
-str_parse_logic <- function(ch, default = TRUE, otherwise = default, n = 1L) {
+str_parse_logic <-
+function(ch, default = TRUE, otherwise = default, n = 1L)
+{
     if (is.na(ch)) default
     else switch(ch,
                 "yes"=, "Yes" =, "true" =, "True" =, "TRUE" = TRUE,
@@ -2230,7 +2335,9 @@ str_parse_logic <- function(ch, default = TRUE, otherwise = default, n = 1L) {
 
 ### ** str_parse
 
-str_parse <- function(ch, default = TRUE, logical = TRUE, otherwise = default, n = 2L) {
+str_parse <-
+function(ch, default = TRUE, logical = TRUE, otherwise = default, n = 2L)
+{
     if(logical)
         str_parse_logic(ch, default=default, otherwise=otherwise, n = n)
     else if(is.na(ch))
