@@ -1,7 +1,7 @@
 #   File src/library/utils/R/Sweave.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -163,8 +163,8 @@ Sweave <- function(file, driver = RweaveLatex(),
                 line <- c(paste0("#line ", linenum+linediff, ' "', basename(file), '"'),
                           line)
             }
-            srclines <- c(attr(chunk, "srclines"), rep(linenum+linediff, length(line)))
-            srcfilenum <- c(attr(chunk, "srcFilenum"), rep(filenum, length(line)))
+            srclines <- c(attr(chunk, "srclines"), rep.int(linenum+linediff, length(line)))
+            srcfilenum <- c(attr(chunk, "srcFilenum"), rep.int(filenum, length(line)))
 	    chunk <- c(chunk, line)
             attr(chunk, "srclines") <- srclines
             attr(chunk, "srcFilenum") <- srcfilenum
@@ -452,10 +452,10 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
     }
     do_exit <-
 	if(no.q)
-	    function(status = 1L) (if(status) stop else message)(
+	    function(status = 0L) (if(status) stop else message)(
 		".Sweave() exit status ", status)
 	else
-	    function(status = 1L) q("no", status = status, runLast = FALSE)
+	    function(status = 0L) q("no", status = status, runLast = FALSE)
 
     if (!length(args)) {
         Usage()
@@ -506,7 +506,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
             compact <- substr(a, 11, 1000)
         } else if (a == "--compact") {
             compact <- "qpdf"
-        } else if (substr(a, 1, 1) == "-") {
+        } else if (startsWith(a, "-")) {
             message(gettextf("Warning: unknown option %s", sQuote(a)),
                     domain = NA)
         } else file <- c(file, a)
@@ -577,16 +577,15 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
     }
     do_exit <-
 	if(no.q)
-	    function(status = 1L) (if(status) stop else message)(
+	    function(status = 0L) (if(status) stop else message)(
 		".Stangle() exit status ", status)
 	else
-	    function(status = 1L) q("no", status = status, runLast = FALSE)
+	    function(status = 0L) q("no", status = status, runLast = FALSE)
 
     if (!length(args)) {
         Usage()
         do_exit(1L)
     }
-
     file <- character()
     encoding <- options <- ""
     engine <- NULL
@@ -612,7 +611,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
             encoding <- substr(a, 12, 1000)
         } else if (substr(a, 1, 10) == "--options=") {
             options <- substr(a, 11, 1000)
-        } else if (substr(a, 1, 1) == "-") {
+        } else if (startsWith(a, "-")) {
             message(gettextf("Warning: unknown option %s", sQuote(a)),
                     domain = NA)
         } else file <- c(file, a)
