@@ -1,7 +1,7 @@
 #  File src/library/utils/R/sessionInfo.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,66 @@
 #  A copy of the GNU General Public License is available at
 #  https://www.R-project.org/Licenses/
 
+.RNGdefaults <- RNGkind() # run once at install time and retain for comparison
+
+## FIXME?  alternatively, just define 'osVersion' directly in .onLoad()  in zzz.R
+.osVersion <- function() {
+    ## Now try to figure out the OS we are running under
+    if (.Platform$OS.type == "windows") {
+        win.version()
+    } else if (nzchar(Sys.which('uname'))) { ## we could try /usr/bin/uname
+        uname <- system("uname -a", intern = TRUE)
+        os <- sub(" .*", "", uname)
+        switch(os,
+               "Linux" =
+                   if(file.exists("/etc/os-release")) {
+                       ## http://www.freedesktop.org/software/systemd/man/os-release.html
+                       tmp <- readLines("/etc/os-release")
+                       t2 <- if (any(startsWith(tmp, "PRETTY_NAME=")))
+                                 sub("^PRETTY_NAME=", "",
+                                     grep("^PRETTY_NAME=", tmp, value = TRUE)[1L])
+                             else if (any(startsWith(tmp, "NAME")))
+                                 ## could check for VERSION or VERSION_ID
+                                 sub("^NAME=", "",
+                                     grep("^NAME=", tmp, value = TRUE)[1L])
+                             else "Linux (unknown distro)"
+                       sub('"(.*)"', "\\1", t2)
+                   } else if(file.exists("/etc/system-release")) {
+                       ## RHEL-like
+                       readLines("/etc/system-release")
+                   },
+               "Darwin" = {
+                   ver <- readLines("/System/Library/CoreServices/SystemVersion.plist")
+                   ind <- grep("ProductUserVisibleVersion", ver)
+                   ver <- ver[ind + 1L]
+                   ver <- sub(".*<string>", "", ver)
+                   ver <- sub("</string>$", "", ver)
+                   ver1 <- strsplit(ver, ".", fixed = TRUE)[[1L]][2L]
+                   sprintf("%s %s %s",
+                           ifelse(as.numeric(ver1) < 12, "OS X", "macOS"),
+                           switch(ver1,
+                                  ## 10.6 is earliest that can be installed
+                                  "6" = "Snow Leopard",
+                                  "7" = "Lion",
+                                  "8" = "Mountain Lion",
+                                  "9" = "Mavericks",
+                                  "10" = "Yosemite",
+                                  "11" = "El Capitan",
+                                  "12" = "Sierra",
+                                  "13" = "High Sierra",
+                                  "14" = "Mojave",
+                                  ""), ver)
+               },
+               "SunOS" = {
+                   ver <- system('uname -r', intern = TRUE)
+                   paste("Solaris",
+                         strsplit(ver, ".", fixed = TRUE)[[1L]][2L])
+               },
+               uname)
+    } # using system('uname ..')
+    ## else NULL
+}
+
 sessionInfo <- function(package = NULL)
 {
     z <- list()
@@ -25,59 +85,8 @@ sessionInfo <- function(package = NULL)
         z$platform <- paste(z$platform, .Platform$r_arch, sep = "/")
     z$platform <- paste0(z$platform, " (", 8*.Machine$sizeof.pointer, "-bit)")
     z$locale <- Sys.getlocale()
-    ## Now try to figure out the OS we are running under
-    if (.Platform$OS.type == "windows") {
-        z$running <- win.version()
-    } else if (nzchar(Sys.which('uname'))) { ## we could try /usr/bin/uname
-        uname <- system("uname -a", intern = TRUE)
-        os <- sub(" .*", "", uname)
-        z$running <-
-            switch(os,
-                   "Linux" = if(file.exists("/etc/os-release")) {
-    ## http://www.freedesktop.org/software/systemd/man/os-release.html
-                       tmp <- readLines("/etc/os-release")
-                       t2 <- if (any(startsWith(tmp, "PRETTY_NAME=")))
-                           sub("^PRETTY_NAME=", "",
-                               grep("^PRETTY_NAME=", tmp, value = TRUE)[1L])
-                       else if (any(startsWith(tmp, "NAME")))
-                           ## could check for VERSION or VERSION_ID
-                           sub("^NAME=", "",
-                               grep("^NAME=", tmp, value = TRUE)[1L])
-                       else "Linux (unknown distro)"
-                       sub('"(.*)"', "\\1", t2)
-                   } else if(file.exists("/etc/system-release")) {
-                       ## RHEL-like
-                       readLines("/etc/system-release")
-                   },
-                   "Darwin" = {
-                       ver <- readLines("/System/Library/CoreServices/SystemVersion.plist")
-                       ind <- grep("ProductUserVisibleVersion", ver)
-                       ver <- ver[ind + 1L]
-                       ver <- sub(".*<string>", "", ver)
-                       ver <- sub("</string>$", "", ver)
-                       ver1 <- strsplit(ver, ".", fixed = TRUE)[[1L]][2L]
-                       sprintf("%s %s %s",
-                               ifelse(as.numeric(ver1) < 12, "OS X", "macOS"),
-                               switch(ver1,
-                                      ## 10.6 is earliest that can be installed
-                                      "6" = "Snow Leopard",
-                                      "7" = "Lion",
-                                      "8" = "Mountain Lion",
-                                      "9" = "Mavericks",
-                                      "10" = "Yosemite",
-                                      "11" = "El Capitan",
-                                      "12" = "Sierra",
-                                      "13" = "High Sierra",
-                                      ""), ver)
-                   },
-                   "SunOS" = {
-                       ver <- system('uname -r', intern = TRUE)
-                       paste("Solaris",
-                             strsplit(ver, ".", fixed = TRUE)[[1L]][2L])
-                   },
-                   uname)
-    }
-
+    z$running <- osVersion
+    z$RNGkind <- RNGkind()
     if(is.null(package)){
         package <- grep("^package:", search(), value=TRUE)
         # weed out environments which are not really packages
@@ -112,7 +121,9 @@ sessionInfo <- function(package = NULL)
     z
 }
 
-print.sessionInfo <- function(x, locale = TRUE, ...)
+print.sessionInfo <- function(x, locale = TRUE,
+			      RNG = !identical(x$RNGkind, .RNGdefaults),
+			      ...)
 {
     mkLabel <- function(L, n) {
         vers <- sapply(L[[n]], function(x) x[["Version"]])
@@ -132,10 +143,17 @@ print.sessionInfo <- function(x, locale = TRUE, ...)
     if (blas == lapack && nzchar(blas))
         cat("BLAS/LAPACK: ", blas, "\n", sep = "")
     else {
-        if (nzchar(blas)) cat("BLAS: ", blas, "\n", sep = "")
-        if (nzchar(lapack)) cat("LAPACK: ", lapack, "\n", sep = "")
+        if(nzchar(blas))   cat("BLAS:   ",   blas, "\n", sep = "")
+        if(nzchar(lapack)) cat("LAPACK: ", lapack, "\n", sep = "")
     }
     cat("\n")
+    if(RNG) {
+        cat("Random number generation:\n"
+          , "RNG:    ", x$RNGkind[1], "\n"
+          , "Normal: ", x$RNGkind[2], "\n"
+          , "Sample: ", x$RNGkind[3], "\n"
+          , "\n")
+    }
     if(locale) {
         cat("locale:\n")
 	print(strsplit(x$locale, ";", fixed=TRUE)[[1]], quote=FALSE, ...)
@@ -163,20 +181,27 @@ toLatexPDlist <- function(pdList, sep = "~") {
     } else ver
 }
 
-toLatex.sessionInfo <- function(object, locale = TRUE, ...)
+toLatex.sessionInfo <-
+    function(object, locale = TRUE,
+	     RNG = !identical(object$RNGkind, .RNGdefaults),
+	     ...)
 {
     z <- c("\\begin{itemize}\\raggedright",
-           paste0("  \\item ", object$R.version$version.string,
-                  ", \\verb|", object$R.version$platform, "|"))
-    if(locale) {
-        z <- c(z,
-               paste0("  \\item Locale: \\verb|",
-                      gsub(";","|, \\\\verb|", object$locale) , "|"))
-    }
-    z <- c(z,
-           paste0("  \\item Running under: \\verb|",
-                  gsub(";","|, \\\\verb|", object$running) , "|"))
-    z <- c(z, paste0("  \\item Matrix products: ", object$matprod))
+	   paste0("  \\item ", object$R.version$version.string,
+		  ", \\verb|", object$R.version$platform, "|"),
+	   if(locale)
+	       paste0("  \\item Locale: \\verb|",
+		      gsub(";","|, \\\\verb|", object$locale) , "|"),
+	   paste0("  \\item Running under: \\verb|",
+		  gsub(";","|, \\\\verb|", object$running) , "|"),
+	   if(RNG)
+	       paste0("  \\item Random number generation:"
+		    , "  \\item RNG:    \\verb|", object$RNGkind[1], "|"
+		    , "  \\item Normal: \\verb|", object$RNGkind[2], "|"
+		    , "  \\item Sample: \\verb|", object$RNGkind[3], "|"
+		      )
+         , paste0("  \\item Matrix products: ", object$matprod)
+           )
     blas <- object$BLAS
     if (is.null(blas)) blas <- ""
     lapack <- object$LAPACK
@@ -186,7 +211,7 @@ toLatex.sessionInfo <- function(object, locale = TRUE, ...)
         z <- c(z, paste0("  \\item BLAS/LAPACK: \\verb|", blas, "|"))
     else {
         if (nzchar(blas))
-            z <- c(z, paste0("  \\item BLAS: \\verb|", blas, "|"))
+            z <- c(z, paste0("  \\item BLAS:   \\verb|", blas, "|"))
         if (nzchar(lapack))
             z <- c(z, paste0("  \\item LAPACK: \\verb|", lapack, "|"))
     }
@@ -197,8 +222,7 @@ toLatex.sessionInfo <- function(object, locale = TRUE, ...)
 
     if(length(o.ver <- toLatexPDlist(object$otherPkg)))
         z <- c(z,
-               strwrap(paste("  \\item Other packages: ",
-			     o.ver),
+               strwrap(paste("  \\item Other packages: ", o.ver),
                        indent = 2, exdent = 4))
     if(length(n.ver <- toLatexPDlist(object$loadedOnly)))
         z <- c(z,
