@@ -1,7 +1,7 @@
 #  File src/library/utils/R/windows/install.packages.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## Unexported helper
 unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
@@ -101,9 +101,8 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
         instPath <- file.path(lib, pkgname)
         if(identical(lock, "pkglock") || isTRUE(lock)) {
             ## This is code adapted from tools:::.install_packages
-            dir.exists <- function(x) !is.na(isdir <- file.info(x)$isdir) & isdir
 	    lockdir <- if(identical(lock, "pkglock"))
-                file.path(lib, paste("00LOCK", pkgname, sep = "-"))
+                file.path(lib, paste0("00LOCK-", pkgname))
             else file.path(lib, "00LOCK")
 	    if (file.exists(lockdir)) {
                 stop(gettextf("ERROR: failed to lock directory %s for modifying\nTry removing %s",
@@ -171,8 +170,8 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
             ret <- unlink(instPath, recursive=TRUE, force=TRUE)
             if (ret == 0) {
                 ## Move the new package to the install lib
-                ## To avoid anti-virus interference, wait a little
-                Sys.sleep(0.5)
+                ## file.rename automatically retries few times if necessary
+		## due to anti-virus interference
                 ret <- file.rename(file.path(tmpDir, pkgname), instPath)
                 if(!ret) {
                     warning(gettextf("unable to move temporary installation %s to %s",
@@ -227,8 +226,10 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
     }
 
     if(is.null(contriburl)) {
-        for(i in seq_along(pkgs))
+        for(i in seq_along(pkgs)) {
+            if(is.na(pkgs[i])) next
             unpackPkgZip(pkgs[i], pkgnames[i], lib, libs_only, lock, quiet)
+        }
         return(invisible())
     }
     tmpd <- destdir
@@ -243,7 +244,7 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
 
     if(is.null(available))
         available <- available.packages(contriburl = contriburl,
-                                        method = method)
+                                        method = method, ...)
     pkgs <- getDependencies(pkgs, dependencies, available, lib, binary = TRUE)
 
     foundpkgs <- download.packages(pkgs, destdir = tmpd, available = available,
@@ -280,8 +281,16 @@ menuInstallPkgs <- function(type = getOption("pkgType"))
 
 menuInstallLocal <- function()
 {
-    install.packages(choose.files('',filters=Filters[c('zip','All'),]),
-                     .libPaths()[1L], repos = NULL)
+    files <- choose.files('',filters=Filters[c('zip','tarball', 'All'),])
+    zips <- endsWith(files, ".zip")
+    tarballs <- endsWith(files, ".tar.gz")
+    bad <- !(zips | tarballs)
+    if (any(bad))
+        stop("Only '*.zip' and '*.tar.gz' files can be installed.")
+    if (any(zips)) install.packages(files[zips],
+        .libPaths()[1L], repos = NULL, type = "binary")
+    if (any(tarballs)) install.packages(files[tarballs],
+        .libPaths()[1L], repos = NULL, type = "source")
 }
 
 ### Deprecated in 2.13.0, defunct in 2.14.0
