@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2001-2015   The R Core Team
+ *  Copyright (C) 2001-2018   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,14 +22,10 @@
 #include <config.h>
 #endif
 
-#define R_USE_SIGNALS 1
-#include <Defn.h>
+#include <Parse.h> // -> IOStuff.h, Defn.h
 #include <Internal.h>
 #include <Fileio.h>
-#include <IOStuff.h>
-#include <Parse.h>
 #include <Rconnections.h>
-#include <IOStuff.h> // for R_ConsoleIob;
 
 SEXP attribute_hidden getParseContext(void)
 {
@@ -177,10 +173,20 @@ void NORET parseError(SEXP call, int linenum)
     UNPROTECT(1);
 }
 
-static void con_cleanup(void *data)
+typedef struct parse_info {
+    Rconnection con;
+    Rboolean old_latin1;
+    Rboolean old_utf8;
+}  parse_cleanup_info;
+
+static void parse_cleanup(void *data)
 {
-    Rconnection con = data;
-    if(con->isopen) con->close(con);
+    parse_cleanup_info *pci = (parse_cleanup_info *)data;
+    Rconnection con = pci->con;
+    if(con && con->isopen)
+	con->close(con);
+    known_to_be_latin1 = pci->old_latin1;
+    known_to_be_utf8 = pci->old_utf8;
 }
 
 /* "do_parse" - the user interface input/output to files.
@@ -192,39 +198,43 @@ static void con_cleanup(void *data)
 */
 SEXP attribute_hidden do_parse(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP text, prompt, s, source;
-    Rconnection con;
-    Rboolean wasopen, old_latin1 = known_to_be_latin1,
-	old_utf8 = known_to_be_utf8, allKnown = TRUE;
-    int ifile, num, i;
-    const char *encoding;
-    ParseStatus status;
-    RCNTXT cntxt;
-
     checkArity(op, args);
     if(!inherits(CAR(args), "connection"))
 	error(_("'file' must be a character string or connection"));
     R_ParseError = 0;
     R_ParseErrorMsg[0] = '\0';
 
-    ifile = asInteger(CAR(args));                       args = CDR(args);
-
-    con = getConnection(ifile);
-    wasopen = con->isopen;
-    num = asInteger(CAR(args));				args = CDR(args);
+    int ifile = asInteger(CAR(args));                   args = CDR(args);
+    Rconnection con = getConnection(ifile);
+    Rboolean wasopen = con->isopen;
+    int num = asInteger(CAR(args));			args = CDR(args);
     if (num == 0)
 	return(allocVector(EXPRSXP, 0));
 
-    PROTECT(text = coerceVector(CAR(args), STRSXP));
+    SEXP text = PROTECT(coerceVector(CAR(args), STRSXP));
     if(length(CAR(args)) && !length(text))
 	error(_("coercion of 'text' to character was unsuccessful"));
     args = CDR(args);
-    prompt = CAR(args);					args = CDR(args);
-    source = CAR(args);					args = CDR(args);
+    SEXP prompt = CAR(args);				args = CDR(args);
+    SEXP source = CAR(args);				args = CDR(args);
     if(!isString(CAR(args)) || LENGTH(CAR(args)) != 1)
 	error(_("invalid '%s' value"), "encoding");
-    encoding = CHAR(STRING_ELT(CAR(args), 0)); /* ASCII */
+    const char *encoding = CHAR(STRING_ELT(CAR(args), 0)); /* ASCII */
+
+    parse_cleanup_info pci;
+    pci.con = NULL;
+    pci.old_latin1 = known_to_be_latin1;
+    pci.old_utf8 = known_to_be_utf8;
+    RCNTXT cntxt;
+    /* set up context to recover known_to_be_* and to close connection on
+       error if opened by do_parse */
+    begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+		 R_NilValue, R_NilValue);
+    cntxt.cend = &parse_cleanup;
+    cntxt.cenddata = &pci;
+
     known_to_be_latin1 = known_to_be_utf8 = FALSE;
+    Rboolean allKnown = TRUE;
     /* allow 'encoding' to override declaration on 'text'. */
     if(streql(encoding, "latin1")) {
 	known_to_be_latin1 = TRUE;
@@ -240,6 +250,8 @@ SEXP attribute_hidden do_parse(SEXP call, SEXP op, SEXP args, SEXP env)
     else
 	PROTECT(prompt = coerceVector(prompt, STRSXP));
 
+    ParseStatus status;
+    SEXP s;
     if (length(text) > 0) {
 	/* If 'text' has known encoding then we can be sure it will be
 	   correctly re-encoded to the current encoding by
@@ -250,15 +262,15 @@ SEXP attribute_hidden do_parse(SEXP call, SEXP op, SEXP args, SEXP env)
 	   different encodings, but all that matters is that all
 	   non-ASCII elements have known encoding.
 	*/
-	for(i = 0; i < length(text); i++)
+	for(int i = 0; i < length(text); i++)
 	    if(!ENC_KNOWN(STRING_ELT(text, i)) &&
 	       !IS_ASCII(STRING_ELT(text, i))) {
 		allKnown = FALSE;
 		break;
 	    }
 	if(allKnown) {
-	    known_to_be_latin1 = old_latin1;
-	    known_to_be_utf8 = old_utf8;
+	    known_to_be_latin1 = pci.old_latin1;
+	    known_to_be_utf8 = pci.old_utf8;
 	}
 	if (num == NA_INTEGER) num = -1;
 	s = R_ParseVector(text, num, &status, source);
@@ -268,17 +280,13 @@ SEXP attribute_hidden do_parse(SEXP call, SEXP op, SEXP args, SEXP env)
 	if (num == NA_INTEGER) num = -1;
 	if(!wasopen) {
 	    if(!con->open(con)) error(_("cannot open the connection"));
-	    /* Set up a context which will close the connection on error */
-	    begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
-			 R_NilValue, R_NilValue);
-	    cntxt.cend = &con_cleanup;
-	    cntxt.cenddata = con;
+	    pci.con = con; /* close the connection on error */
 	}
 	if(!con->canread) error(_("cannot read from this connection"));
 	s = R_ParseConn(con, num, &status, source);
 	if(!wasopen) {
 	    PROTECT(s);
-	    endcontext(&cntxt);
+	    pci.con = NULL;
 	    con->close(con);
 	    UNPROTECT(1);
 	}
@@ -289,8 +297,10 @@ SEXP attribute_hidden do_parse(SEXP call, SEXP op, SEXP args, SEXP env)
 	s = R_ParseBuffer(&R_ConsoleIob, num, &status, prompt, source);
 	if (status != PARSE_OK) parseError(call, R_ParseError);
     }
-    UNPROTECT(2);
-    known_to_be_latin1 = old_latin1;
-    known_to_be_utf8 = old_utf8;
+    known_to_be_latin1 = pci.old_latin1;
+    known_to_be_utf8 = pci.old_utf8;
+    PROTECT(s);
+    endcontext(&cntxt);
+    UNPROTECT(3);
     return s;
 }
