@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001--2013  The R Core Team.
+ *  Copyright (C) 2001--2018  The R Core Team.
  *  Copyright (C) 2003--2010  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /* Interface routines, callable from R using .Internal, for Lapack code */
@@ -25,13 +25,31 @@
 #endif
 
 #include <Defn.h>
-#include <ctype.h> /* for toupper */
+#include <ctype.h>  /* for toupper */
+#include <limits.h> /* for PATH_MAX */
+#include <stdlib.h> /* for realpath */
+
+#ifdef HAVE_UNISTD_H
+# include <unistd.h> /* for realpath on some systems */
+#endif
+
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>  /* for dladdr */
+#endif
+
+#if defined(HAVE_REALPATH) && defined(HAVE_DECL_REALPATH) && !HAVE_DECL_REALPATH
+extern char *realpath(const char *path, char *resolved_path);
+#endif
+
+#if defined(HAVE_DLADDR) && defined(HAVE_DECL_DLADDR) && !HAVE_DECL_DLADDR
+extern int dladdr(void *addr, Dl_info *info);
+#endif
 
 #include "Lapack.h"
 
 /* NB: the handling of dims is odd here.  Most are coerced to be
  * integers (which dimgets currently guarantees), but a couple were
- * used unchecked. 
+ * used unchecked.
  */
 
 /* FIXME: MM would want to make these available to packages;
@@ -78,29 +96,26 @@ static char La_rcond_type(const char *typstr)
 /* La.svd, called from svd */
 static SEXP La_svd(SEXP jobu, SEXP x, SEXP s, SEXP u, SEXP vt)
 {
-    int n, p, info = 0;
-
     if (!isString(jobu))
 	error("'jobu' must be a character string");
-    int *xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
-    n = xdims[0]; p = xdims[1];
+    int *xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP)),
+	n = xdims[0], p = xdims[1], nprot = 2;
 
     /* work on a copy of x  */
     double *xvals;
     if (!isReal(x)) {
-       x = coerceVector(x, REALSXP);
-       xvals = REAL(x);
+	x = PROTECT(coerceVector(x, REALSXP)); nprot++;
+	xvals = REAL(x);
     } else {
 	xvals = (double *) R_alloc(n * (size_t) p, sizeof(double));
 	Memcpy(xvals, REAL(x), n * (size_t) p);
     }
-    PROTECT(x);
 
     SEXP dims = getAttrib(u, R_DimSymbol);
-    if (TYPEOF(dims) != INTSXP) error("non-integer dims");
+    if (TYPEOF(dims) != INTSXP) error("non-integer dim(u)");
     int ldu = INTEGER(dims)[0];
     dims = getAttrib(vt, R_DimSymbol);
-    if (TYPEOF(dims) != INTSXP) error("non-integer dims");
+    if (TYPEOF(dims) != INTSXP) error("non-integer dim(vt)");
     int ldvt = INTEGER(dims)[0];
     double tmp;
     /* min(n,p) large is implausible, but cast to be sure */
@@ -108,7 +123,7 @@ static SEXP La_svd(SEXP jobu, SEXP x, SEXP s, SEXP u, SEXP vt)
 
     /* ask for optimal size of work array */
     const char *ju = CHAR(STRING_ELT(jobu, 0));
-    int lwork = -1;
+    int info = 0, lwork = -1;
     F77_CALL(dgesdd)(ju, &n, &p, xvals, &n, REAL(s),
 		     REAL(u), &ldu, REAL(vt), &ldvt,
 		     &tmp, &lwork, iwork, &info);
@@ -131,7 +146,7 @@ static SEXP La_svd(SEXP jobu, SEXP x, SEXP s, SEXP u, SEXP vt)
     SET_VECTOR_ELT(val, 0, s);
     SET_VECTOR_ELT(val, 1, u);
     SET_VECTOR_ELT(val, 2, vt);
-    UNPROTECT(3);
+    UNPROTECT(nprot);
     return val;
 }
 
@@ -198,7 +213,6 @@ static SEXP La_rs(SEXP x, SEXP only_values)
 	nm = PROTECT(allocVector(STRSXP, 2));
 	SET_STRING_ELT(nm, 1, mkChar("vectors"));
 	SET_VECTOR_ELT(ret, 1, z);
-	UNPROTECT_PTR(z);
     } else {
 	ret = PROTECT(allocVector(VECSXP, 1));
 	nm = PROTECT(allocVector(STRSXP, 1));
@@ -206,7 +220,7 @@ static SEXP La_rs(SEXP x, SEXP only_values)
     SET_STRING_ELT(nm, 0, mkChar("values"));
     setAttrib(ret, R_NamesSymbol, nm);
     SET_VECTOR_ELT(ret, 0, values);
-    UNPROTECT(4);
+    UNPROTECT(ov ? 4 : 5);
     return ret;
 }
 
@@ -558,7 +572,7 @@ static SEXP La_solve_cmplx(SEXP A, SEXP Bin)
 	p = 1;
 	if(length(Bin) != n)
 	    error(_("'b' (%d x %d) must be compatible with 'a' (%d x %d)"),
-		  length(Bin), p, n, n);	
+		  length(Bin), p, n, n);
 	PROTECT(B = allocVector(CPLXSXP, n));
 	if (!isNull(Adn)) setAttrib(B, R_NamesSymbol, VECTOR_ELT(Adn, 1));
     }
@@ -824,7 +838,7 @@ static SEXP La_rs_cmplx(SEXP xin, SEXP only_values)
     SEXP values = PROTECT(allocVector(REALSXP, n));
     rvalues = REAL(values);
 
-    rwork = (double *) R_alloc((3*(size_t)n-2) > 1 ? 3*(size_t)n-2 : 1, 
+    rwork = (double *) R_alloc((3*(size_t)n-2) > 1 ? 3*(size_t)n-2 : 1,
 			       sizeof(double));
     /* ask for optimal size of work array */
     lwork = -1;
@@ -970,7 +984,7 @@ static SEXP La_chol(SEXP A, SEXP pivot, SEXP stol)
 	    // need to pivot the colnames
 	    SEXP dn2 = PROTECT(duplicate(dn));
 	    SEXP cn2 = VECTOR_ELT(dn2, 1);
-	    for(int i = 0; i < m; i++) 
+	    for(int i = 0; i < m; i++)
 		SET_STRING_ELT(cn2, i, STRING_ELT(cn, ip[i] - 1)); // base 1
 	    setAttrib(ans, R_DimNamesSymbol, dn2);
 	    UNPROTECT(1);
@@ -1035,7 +1049,7 @@ static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
     double *avals, anorm, rcond, tol = asReal(tolin), *work;
     SEXP B, Adn, Bdn;
 
-    if (!(isMatrix(A) && 
+    if (!(isMatrix(A) &&
 	  (TYPEOF(A) == REALSXP || TYPEOF(A) == INTSXP || TYPEOF(A) == LGLSXP)))
 	error(_("'a' must be a numeric matrix"));
     int *Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
@@ -1068,13 +1082,13 @@ static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
 	p = 1;
 	if(length(Bin) != n)
 	    error(_("'b' (%d x %d) must be compatible with 'a' (%d x %d)"),
-		  length(Bin), p, n, n);	
+		  length(Bin), p, n, n);
 	PROTECT(B = allocVector(REALSXP, n));
 	if (!isNull(Adn)) setAttrib(B, R_NamesSymbol, VECTOR_ELT(Adn, 1));
     }
     PROTECT(Bin = coerceVector(Bin, REALSXP));
     Memcpy(REAL(B), REAL(Bin), (size_t)n * p);
-    
+
     int *ipiv = (int *) R_alloc(n, sizeof(int));
 
     /* work on a copy of A */
@@ -1167,33 +1181,31 @@ static SEXP La_qr(SEXP Ain)
 /* Real case of qr.coef */
 static SEXP qr_coef_real(SEXP Q, SEXP Bin)
 {
-    int n, nrhs, k, *Bdims;
-    SEXP B, qr = VECTOR_ELT(Q, 0), tau = VECTOR_ELT(Q, 2);
-    double *work, tmp;
-
-    k = LENGTH(tau);
     if (!isMatrix(Bin)) error(_("'b' must be a numeric matrix"));
-    
-    B = PROTECT(isReal(Bin) ? duplicate(Bin) : coerceVector(Bin, REALSXP));
 
-    n = INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP))[0];
-    Bdims = INTEGER(coerceVector(getAttrib(B, R_DimSymbol), INTSXP));
+    SEXP B = PROTECT(isReal(Bin) ? duplicate(Bin) : coerceVector(Bin, REALSXP)),
+	qr  = VECTOR_ELT(Q, 0), // qr$qr
+	tau = VECTOR_ELT(Q, 2); // qr$qraux
+    int k = LENGTH(tau),
+	n =      INTEGER(coerceVector(getAttrib(qr, R_DimSymbol), INTSXP))[0],
+	*Bdims = INTEGER(coerceVector(getAttrib(B,  R_DimSymbol), INTSXP));
     if(Bdims[0] != n)
 	error(_("right-hand side should have %d not %d rows"), n, Bdims[0]);
-    nrhs = Bdims[1];
-    int lwork = -1, info;
+    int nrhs = Bdims[1],
+	lwork = -1, info;
+    double tmp;
     F77_CALL(dormqr)("L", "T", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     &tmp, &lwork, &info);
     if (info != 0)
-	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr [tmp]");
     lwork = (int) tmp;
-    work = (double *) R_alloc(lwork, sizeof(double));
+    double *work = (double *) R_alloc(lwork, sizeof(double));
     F77_CALL(dormqr)("L", "T", &n, &nrhs, &k,
 		     REAL(qr), &n, REAL(tau), REAL(B), &n,
 		     work, &lwork, &info);
     if (info != 0)
-	error(_("error code %d from Lapack routine '%s'"), info, "dormqr");
+	error(_("error code %d from Lapack routine '%s'"), info, "dormqr [work]");
     F77_CALL(dtrtrs)("U", "N", "N", &k, &nrhs,
 		     REAL(qr), &n, REAL(B), &n, &info);
     if (info != 0)
@@ -1356,8 +1368,26 @@ static SEXP mod_do_lapack(SEXP call, SEXP op, SEXP args, SEXP env)
 	ans = mkString(str);
 	break;
     }
+    case 1001:
+    {
+#if defined(HAVE_DLADDR) && defined(HAVE_REALPATH)
+	Dl_info dl_info;
+	// dladdr has first arg void * on Solaris.  This is not POSIX.
+	if (dladdr((void *) F77_NAME(ilaver), &dl_info)) {
+	    char buf[PATH_MAX+1];
+	    char *res = realpath(dl_info.dli_fname, buf);
+	    if (res) {
+		ans = mkString(res);
+		break;
+	    }
+	}
+#endif
+	ans = mkString(""); /* LAPACK library not known */
+	break;
     }
 
+    }// switch
+
     return ans;
 }
 
