@@ -91,10 +91,12 @@ void attribute_hidden Rstd_Suicide(const char *s)
 /*
   The following provides a version of select() that catches interrupts
   and handles them using the supplied interrupt handler or the default
-  one if NULL is supplied.  The interrupt handler must exit using a
-  longjmp.  If the supplied timout value os zero, select is called
-  without setting up an error handler since it should return
-  immediately.
+  one if NULL is supplied.  The interrupt handler can return,
+  e.g. after invoking a resume restart. If the interrupt handler
+  returns then the select call is retried. If the timeout is not NULL
+  then the timeout is adjusted for the elapsed time before the retry.
+  If the supplied timout value is zero, select is called without
+  setting up an error handler since it should return immediately.
  */
 
 static SIGJMP_BUF seljmpbuf;
@@ -113,19 +115,44 @@ int R_SelectEx(int  n,  fd_set  *readfds,  fd_set  *writefds,
 	       fd_set *exceptfds, struct timeval *timeout,
 	       void (*intr)(void))
 {
+    /* FD_SETSIZE should be at least 1024 on all supported
+       platforms. If this still turns out to be limiting we will
+       probably need to rewrite internals to use poll() instead of
+       select().  LT */
+    if (n > FD_SETSIZE)
+	error("file descriptor is too large for select()");
+
     if (timeout != NULL && timeout->tv_sec == 0 && timeout->tv_usec == 0)
-	/* Is it right for select calls with a timeout to be
-	   non-interruptable? LT */
 	return select(n, readfds, writefds, exceptfds, timeout);
     else {
 	volatile sel_intr_handler_t myintr = intr != NULL ?
-	    intr : onintrNoResume;
+	    intr : onintr;
 	volatile int old_interrupts_suspended = R_interrupts_suspended;
+	volatile double base_time = currentTime();
+	struct timeval tm;
+	if (timeout != NULL)
+	    tm = *timeout;
+    retry:
 	if (SIGSETJMP(seljmpbuf, 1)) {
 	    myintr();
-	    R_interrupts_suspended = old_interrupts_suspended;
-	    error(_("interrupt handler must not return"));
-	    return 0; /* not reached */
+
+	    if (timeout != NULL) {
+		/* Ajdust timeout for elapsed complete seconds; ignore
+		   microseconde for now. This modifies the data pointed to
+		   by timeval, which is what select() on Linux does as
+		   well. */
+		double new_time = currentTime();
+		double elapsed = new_time = base_time;
+		base_time = new_time;
+		time_t elapsed_sec = (time_t) elapsed;
+		if (tm.tv_sec > elapsed_sec)
+		    tm.tv_sec -= elapsed_sec;
+		else
+		    tm.tv_sec = 0;
+		*timeout = tm;
+	    }
+
+	    goto retry;
 	}
 	else {
 	    int val;
@@ -563,7 +590,11 @@ pushReadline(const char *prompt, rl_vcpfunc_t f)
    rl_callback_handler_install(prompt, f);
 
 #ifdef NEED_INT_HANDLER
-   signal(SIGWINCH, R_readline_sigwinch_handler);
+    struct sigaction sa;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_handler = &R_readline_sigwinch_handler;
+    sa.sa_flags = SA_RESTART;
+    sigaction(SIGWINCH, &sa, NULL);
 #endif
 
    /* flush stdout in case readline wrote the prompt, but didn't flush
@@ -947,9 +978,13 @@ Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
 	    *ob = '\0';
 	    err = res == (size_t)(-1);
 	    /* errors lead to part of the input line being ignored */
-	    if(err) printf(_("<ERROR: re-encoding failure from encoding '%s'>\n"),
-			   R_StdinEnc);
-	    strncpy((char *)buf, obuf, len);
+	    if(err) {
+		printf(_("<ERROR: re-encoding failure from encoding '%s'>\n"),
+		       R_StdinEnc);
+		strncpy((char *)buf, obuf, len);
+		strcat((char *)buf, "...\n");
+	    } else
+		strncpy((char *)buf, obuf, len);
 	}
 /* according to system.txt, should be terminated in \n, so check this
    at eof and error */
@@ -1005,6 +1040,18 @@ Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
 		// introduced in readline 4.0: only used for >= 6.3
 #ifdef HAVE_RL_RESIZE_TERMINAL
 		rl_resize_terminal();
+		static int oldwidth;
+		int height, width;
+		rl_get_screen_size(&height,&width);
+		if (oldwidth >= 0 && oldwidth != width) {
+		    static SEXP opsym = NULL;
+		    if (! opsym)
+			opsym = install("setWidthOnResize");
+		    Rboolean setOK = asLogical(GetOption1(opsym));
+		    oldwidth = width;
+		    if (setOK != NA_LOGICAL && setOK)
+			R_SetOptionWidth(width);
+		}
 #endif
             }
 #endif
@@ -1113,11 +1160,12 @@ void R_CleanTempDir(void)
     char buf[1024];
 
     if((Sys_TempDir)) {
-#if defined(sun) || defined(__sun)
+// Only __sun is neeed on Solaris >= 10 (2005).
+#if defined(__sun) || defined(sun)
 	/* On Solaris the working directory must be outside this one */
 	chdir(R_HomeDir());
 #endif
-	snprintf(buf, 1024, "rm -rf %s", Sys_TempDir);
+	snprintf(buf, 1024, "rm -Rf %s", Sys_TempDir);
 	buf[1023] = '\0';
 	R_system(buf);
     }
@@ -1252,6 +1300,8 @@ Rstd_ShowFiles(int nfile,		/* number of files */
 	}
 	snprintf(buf, 1024, "'%s' < '%s'", pager, filename); //might contain spaces
 	res = R_system(buf);
+	if (res == 127)
+	    warningcall(R_NilValue, _("error in running command"));
 	unlink(filename);
 	free(filename);
 	return (res != 0);
