@@ -103,29 +103,25 @@ show(foo)
 ## show() method for this class.  A better solution would be desirable.
 print(foo, digits = 4)
 
-setClassUnion("integer or NULL", members = c("integer","NULL"))
-setClass("c1", representation(x = "integer", code = "integer or NULL"))
+cn <- "integer or NULL"
+setClassUnion(cn, members = c("integer", "NULL"))
+setClass("c1", representation(x = "integer", code = cn))
+stopifnot(exprs = {
+    cn %in% extends(getClass("NULL"))
+    cn %in% extends(getClass(".NULL"))
+    cn %in% extends(getClass("integer"))
+})
 nc <- new("c1", x = 1:2)
 str(nc)# gave ^ANULL^A in 2.0.0
 ##
 
-
-library(stats4)
 showMethods("coerce", classes=c("matrix", "numeric"))
 ## {gave wrong result for a while in R 2.4.0}
 
-## the following showMethods() output tends to generate errors in the tests
-## whenever the contents of the packages change. Searching in the
-## diff's can easily mask real problems.  If there is a point
-## to the printout, e.g., to verify that certain methods exist,
-## hasMethod() would be a useful replacement
-
-## showMethods(where = "package:stats4")
-## showMethods("show")
-## showMethods("show")
-## showMethods("plot") # (ANY,ANY) and (profile.mle, missing)
-## showMethods(classes="mle")
-## showMethods(classes="matrix")
+## Most for "mle" in stats4:
+for(f in c("coef", "confint", "logLik", "plot", "profile",
+	   "show", "summary", "update", "vcov"))
+    if(!hasMethods(f)) stop("no S4 methods found for ", f)
 
 
 ##--- "[" fiasco before R 2.2.0 :
@@ -607,13 +603,16 @@ if( identical(f, L$A) )
 
 
 ## prototypes for virtual classes:  NULL if legal, otherwise 1st member
-## OptionalPosixct above includes NULL
-stopifnot(is.null(getClass("OptionalPOSIXct")@prototype))
 ## "IntOrChar" had invalid NULL prototype < 2.15.0
 setClassUnion("IntOrChar", c("integer", "character"))
-stopifnot(is.integer(getClass("IntOrChar")@prototype))
-## produced an error < 2.15.0
-stopifnot(identical(isGeneric("&&"), FALSE))
+stopifnot(exprs = {
+    ## OptionalPosixct above includes NULL
+    is.null   (getClass("OptionalPOSIXct")@prototype)
+    is.integer(getClass("IntOrChar")      @prototype) ## produced an error < 2.15.0
+    "IntOrChar" %in% extends(getClass("character"))
+    "IntOrChar" %in% extends(getClass("integer"))
+    identical(isGeneric("&&"), FALSE)
+})
 
 
 ## mapply() on S4 objects with a "non-primitive" length() method
@@ -905,3 +904,72 @@ setMethod("[", "bar", function(x, i, j, ..., flag = FALSE, drop = FALSE) {
 BAR <- new("bar")
 stopifnot(identical(BAR[1L], FALSE))
 stopifnot(identical(BAR[1L, , flag=TRUE], TRUE))
+
+## avoid infinite recursion on Ops,structure methods
+setClass("MyInteger",
+         representation("integer")
+         )
+i <- new("MyInteger", 1L)
+m <- matrix(rnorm(300), 30,10)
+stopifnot(identical(i*m, m))
+
+## when rematching, do not drop arg with NULL default
+setGeneric("genericExtraArg",
+           function(x, y, extra) standardGeneric("genericExtraArg"),
+           signature="x")
+
+setMethod("genericExtraArg", "ANY", function(x, y=NULL) y)
+
+stopifnot(identical(genericExtraArg("foo", 1L), 1L))
+
+## callNextMethod() was broken for ... dispatch
+f <- function(...) length(list(...))
+setGeneric("f")
+setMethod("f", "character", function(...){ callNextMethod() })
+stopifnot(identical(f(1, 2, 3), 3L))
+stopifnot(identical(f("a", "b", "c"), 3L))
+
+## ... dispatch was evaluating missing arguments in the generic frame
+f <- function(x, ..., a = b) {
+    b <- "a"
+    a
+}
+setGeneric("f", signature = "...")
+stopifnot(identical(f(a=1), 1))
+stopifnot(identical(f(), "a"))
+
+## ensure forwarding works correctly for dots dispatch
+f2 <- function(...) f(...)
+stopifnot(identical(f2(a=1), 1))
+
+
+## R's internal C  R_check_class_and_super()  was not good enough
+if(require("Matrix")) withAutoprint({
+    setClass("Z", representation(zz = "list"))
+    setClass("C", contains = c("Z", "dgCMatrix"))
+    setClass("C2", contains = "C")
+    setClass("C3", contains = "C2")
+    m <- matrix(c(0,0,2:0), 3,5, dimnames = list(NULL,NULL))
+    (mC <- as(m, "dgCMatrix"))
+    (cc <- as(mC, "C"))
+     c2 <- as(mC, "C2")
+     c3 <- as(mC, "C3")
+    stopifnot(
+        identical(capture.output(c2),
+                  sub("C3","C2", capture.output(c3)))
+      , identical(as(cc, "matrix"), m)
+      , identical(as(c2, "matrix"), m)
+      , identical(as(c3, "matrix"), m)
+    )
+    invisible(lapply(c("Z","C","C2","C3"), removeClass))
+})
+
+
+## Automatic coerce method creation:
+setClass("A", slots = c(foo = "numeric"))
+setClass("Ap", contains = "A", slots = c(p = "character"))
+cd <- getClassDef("Ap")
+body(cd@contains[["A"]]@coerce)[[2]] ## >>   value <- new("A")
+## was ... <-  new(structure("A", package = ".GlobalEnv"))
+## for a few days in R-devel (Nov.2017)
+
