@@ -147,7 +147,7 @@ stopifnot(p > 0.001)
 Dates <- seq(as.Date("2005/01/01"), as.Date("2009/01/01"), "day")
 months <- format(Dates, format = "%m")
 years <- format(Dates, format = "%Y")
-mn <- as.vector(unlist(sapply(split(months, years), table)))
+mn <- unlist(lapply(unname(split(months, years)), table), use.names=FALSE)
 ty <- as.vector(table(years))
 # Test hist.Date() for months
 stopifnot(identical(hist(Dates, "month", plot = FALSE)$counts, mn))
@@ -188,7 +188,8 @@ tapply(character(0), factor(letters)[FALSE], length)
 
 
 ## zero-length patterns in gregexpr
-expect <- structure(1:3, match.length=rep(0L, 3), useBytes = TRUE)
+expect <- structure(1:3, match.length=rep(0L, 3),
+                    index.type = "chars", useBytes = TRUE)
 stopifnot(identical(expect, gregexpr("", "abc")[[1]]))
 stopifnot(identical(expect, gregexpr("", "abc", fixed=TRUE)[[1]]))
 stopifnot(identical(expect, gregexpr("", "abc", perl=TRUE)[[1]]))
@@ -231,6 +232,7 @@ op <- options(warn=2)
 m <- c(-Inf,Inf)
 stopifnot(rnorm(2, mean = m) == m,
           rexp (2, Inf) == 0)
+set.seed(11)
 rt(1, Inf)
 R <- list(try(rnorm(2, numeric())),
           try(rexp (2, numeric())),
@@ -243,6 +245,7 @@ stopifnot(sapply(R, function(ch) sub(".* : ", '', ch) ==
 
 
 ## predict.loess with transformed variables
+set.seed(11)
 y <- 1:100 + rnorm(100)
 od <- data.frame(x=1:100, z=1:100 + rnorm(100, 10))
 nd <- data.frame(x=1:100, z=11:110)
@@ -294,11 +297,14 @@ dimnames(n)[[1]] <- c("a", "b")
 ## glob2rx(pattern, .) with "(", "[" or "{" in pattern :
 nm <- "my(ugly[file{name"
 stopifnot(identical(regexpr(glob2rx("*[*"), nm),
-		    structure(1L, match.length = 8L, useBytes = TRUE)),
+		    structure(1L, match.length = 8L,
+                              index.type = "chars", useBytes = TRUE)),
 	  identical(regexpr(glob2rx("*{n*"), nm),
-		    structure(1L, match.length = 14L, useBytes = TRUE)),
+		    structure(1L, match.length = 14L,
+                              index.type = "chars", useBytes = TRUE)),
 	  identical(regexpr(glob2rx("*y(*{*"), nm),
-		    structure(1L, match.length = 13L, useBytes = TRUE))
+		    structure(1L, match.length = 13L,
+                              index.type = "chars", useBytes = TRUE))
 	  )
 ## gave 'Invalid regular expression' in R <= 2.7.0
 
@@ -321,7 +327,7 @@ stopifnot(identical(unclass(z), c('z', 'y', 'x', 'w')))
 ## repeated third and later args in R < 2.7.1.
 
 ## PD found that f==f contains NA when f has NA levels (but no missing value)
-f1 <- factor(c(1, 2, NA), exclude = "")
+f1 <- factor(c(1, 2, NA), levels = 1:2)
 f2 <- factor(c(1, 2, NA), exclude = NULL)
 stopifnot(identical(f1, factor(c(1,2,NA))),
           nlevels(f1) == 2, nlevels(f2) == 3,
@@ -717,11 +723,11 @@ stopifnot(length(lf <- levels(fi)) == 3, lf[1] == "a.b.c",
 ## interaction() failed to produce unique levels before 2.9.1
 
 levs <- c("A","A")
-## warnings for now {errors in the future}
+## warnings since 2009; errors since R 3.4.0 (R-devel, June 2016):
 local({
-    assertWarning(gl(2,3, labels = levs))
-    assertWarning(factor(levs, levels=levs))
-    assertWarning(factor(1:2,	 labels=levs))
+    assertError(gl(2,3, labels = levs))
+    assertError(factor(levs, levels=levs))
+    ## now valid: factor(1:2,  labels=levs)
     })
 ## failed in R < 2.10.0
 L <- c("no", "yes")
@@ -880,7 +886,7 @@ m1 <- methods(na.omit) ## should give (no warning):
 setClass("bla")
 setMethod("na.omit", "bla", function(object, ...) "na.omit(<bla>)")
 (m2 <- methods(na.omit)) ## should give (no warning):
-stopifnot(identical(m1, m2))
+stopifnot(identical(m1, .S3methods("na.omit")))
 options(op)
 ## gave two warnings, when an S3 generic had turned into an S4 one
 
@@ -1811,13 +1817,14 @@ d <- dist(matrix(round(rnorm(n*p), digits = 2), n,p), "manhattan")
 d[] <- d[] * sample(1 + (-4:4)/100, length(d), replace=TRUE)
 hc <- hclust(d, method = "median")
 stopifnot(all.equal(hc$height[5:11],
-                    c(1.69805, 1.75134375, 1.34036875, 1.47646406,
-                      3.21380039, 2.9653438476, 6.1418258), tolerance = 1e-9))
+		    c(1.70595, 1.657675, 1.8909, 1.619973438, 
+                      1.548624609, 3.097474902, 6.097159351),
+                    tolerance = 1e-9))
 ## Also ensure that hclust() remains fast:
 set.seed(1); nn <- 2000
-tm0 <- system.time(dst <- as.dist(matrix(runif(n = nn^2, min = 0, max = 1), nn, nn)))
+tm0 <- system.time(dst <- as.dist(matrix(runif(n = nn^2, min = 0, max = 1)^1.1, nn, nn)))
 (tm <- system.time(hc <- hclust(dst, method="average")))
-stopifnot(tm[1] < tm0[1])
+stopifnot(tm[1] <= tm0[1])
 ## was slow  from R 1.9.0 up to R 2.15.0
 
 
@@ -2072,7 +2079,7 @@ stopifnot(!is.na(z$tension))
 p <- file.path(R.home("share"),"texmf") # always exists, readable
 lfri <- list.files(p, recursive=TRUE, include.dirs=TRUE)
 subdirs <- c("bibtex", "tex")
-lfnd <- setdiff(list.files(p, all.files=TRUE, no..=TRUE), ".svn")
+lfnd <- setdiff(list.files(p, all.files=TRUE, no..=TRUE), c(".svn", ".DS_Store"))
 stopifnot(!is.na(match(subdirs, lfri)), identical(subdirs, lfnd))
 ## the first failed for a few days, unnoticed, in the development version of R
 
@@ -2095,14 +2102,7 @@ assertError( aperm(a, "A"))
 stopifnot(identical(NA_character_, enc2utf8(NA_character_)))
 ## gave "NA" instead of NA_character_
 
-## as.hexmode(x), as.octmode(x)  when x is double
-x <- c(NA, 1)
-stopifnot(identical(x == x,
-		    as.hexmode(x) == as.octmode(x)))
-p <- c(1, pi)
-assertError(as.hexmode(p))
-assertError(as.octmode(p))
-## where all "wrong" in R <= 3.1.1
-
+## End of regression tests for R < 3.0.0
+## -------------------------------------
 
 proc.time()
